

# This class specifies the parameters used during the Monte Carlo optimization
class MonteCarloParams:

    def __init__(self):

        # Profile of the set of temperatures used. You can add as many as you
        # want. You can see that here the profile is a tempering
        self.temperatures = [30000, 15000, 10000, 5000, 1000]
        # Monte Carlo iterations to use for each value of the temperature.
        # The number of values must be the same as the number of temperatures.
        self.iterations = [200, 200, 200, 200]
        # Maximum translation allowed for the components for each value of
        # the temperature
        self.max_translations = [20, 15, 10, 5]
        # Same as above but with the rotations (in radians)
        self.max_rotations = [1, 0.5, 0.25, 0.1]
        # The number of cycles. Is the number of times that the profile
        # of temperatures, iterations, translations and rotations is repeated
        # Yo can put as many as you want
        self.cycles = 3
        # This parameter indicates the probability for a component of doing
        #  a random movement instead of a relative movement. A value of 0.4
        # means that a component will do a
        # random move instead of moving to a relative position
        # respect to its receptor 40% of the time.
        # If no docking has been done between two structures, this parameter is
        # ignored.
        self.non_relative_move_prob = 0.4


# This class specifies the parameters used for the DOMINO sampling
class DominoSamplingPositions:

    def __init__(self):
        # This parameter is the file where the results of many different
        # Monte Carlo runs are. The starting solutions for DOMINO are read from
        # it. It must be in the SQLite database format, as generated by the
        # MC sampling.
        self.read = "monte_carlo_solutions.db"
        # The maximum number of solutions from the file in self.read to combine
        self.max_number = 50
        # criteria to order the solutions
        self.orderby = "em2d"
        # Here, all the parameters together mean that the 50 best solutions from monte_carlo_solutions.db according to the em2d score are fed to
        # DOMINO.

# This class is for technical parameters for DOMINO.


class DominoParams:

    def __init__(self, ):
        # Exploring all possible combinations of sampling positions
        # is memory intensive. self.heap_solutions is the number of
        #  solutions that you keep each merging step in DOMINO.
        # The larger the number, the better the space of combinations
        #  is explored, at the cost of larger running time
        self.heap_solutions = 2000

# This class is here to inject testing code in the modeling. Don't bother with
# it unless you know the code for the application well.


class TestParameters(object):

    def __init__(self, ):
        self.do_test = False
        self.test_fn_assembly = ""


# This class is used to measure the performance of the method with benchmark
# structures. Same as above, ignore or delete it if you are modeling.
class Benchmark:

    def __init__(self, ):
        self.fn_pdb_native = "3sfd.pdb"
        # parameters for measuring the ccc with the 3D map
        self.resolution = 8
        self.voxel_size = 1.5
        self.native_map_threshold = 0.25
        # The component from the model is aligned to the native component before
        # computing the measures for the benchmark
        self.native_component = "3sfdB"
        self.model_component = "3sfdB"


# this is the main class containing all the parameters of the experiment
class Experiment (object):

    def __init__(self,):

        # The name of the model. You use any name. It is not used by the logic
        # of the application
        self.model_name = ["3sfd"]
        # The name of each of the components (subunits) of the assembly. The
        # names of the subunits are heavily used by the code. Make sure that all
        # the names are different.
        self.names = ["3sfdA", "3sfdB", "3sfdC", "3sfdD"]
        # The number of the pdbs containing the subunits. The name of the
        # files does not need to match the names of the subunits in self.names
        self.fn_pdbs = ["3sfdA.pdb", "3sfdB.pdb", "3sfdC.pdb", "3sfdD.pdb"]
        # Some of the restraints (e.g., excluded volume), use a coarse
        # representation of the components of the assembly. This value is
        # the number of residues included in a bead of the coarse
        # representation
        self.n_residues = 10
        # The components that are going to be anchored. i.e., the components
        # that are going to be set at the origin and with a identity rotation.
        # First start with all False until you perform the initial dockings of
        # components, and the anchor the first receptor. You can override the
        # selection of the anchored component if you want. The only condition
        # is that the anchor can not move relative to other components.
        # The program will crash otherwise.
        self.anchor = [False, False, False, False]
        # This option means that the coordinates of a component are fixed.
        # It makes sense when using DOMINO. It keeps a component in its
        # actual position, ignoring all the possible positions obtained using
        # the Monte Carlo modelings.
        self.fixed = [False, False, False, False]

        # Pair score restraint. This restraint indicates that 2 components of
        # the complex should be together. The order of the parameters for each
        # restraint is:
        #  component1 - the id of the first component
        #  component2 - the id of the second component
        #  name - The name for the restraint. Make sure that the restraint
        #       does not contain hyphens. SQLite does not like them. Do not
        # given the same name to different restraints.
        #  distance - The distance that the components should keep. It is the
        #       minimum distance between 2 particles of the complex. Distances
        # equal or below this value are not penalized by the scoring function
        # Weight - the weight for the restraint. The value of a restraint is
        #       multiplied by its weight before adding it to the total scoring
        #       function.
        # stddev - This is the "standard deviation". The pair score restraint
        # is an IMP.HarmonicUppperBound function. The stddev value is used to
        # approximate the values of the HarmonicUppperBound to those of a
        # Gaussian function of standard deviation stddev.

        # component1,component2,name,distance,weight,pairs,stddev
        self.pair_score_restraints = [["3sfdB", "3sfdA", "B_A", 0, 1, 1, 3.],
                                      ["3sfdB", "3sfdC", "B_C", 0, 1, 1, 3.],
                                      ["3sfdB", "3sfdD", "B_D", 0, 1, 1, 3.],
                                      ["3sfdD", "3sfdC", "D_C", 0, 1, 1, 3.]

                                      ]

        # Excluded volume restraint. Restraint preventing the components from
        # interpenetrating. This restraints sets an all agains all excluded
        # volument restraint. The meaning of the parameters is the same as above
        #distance, weight,pairs,stddev
        self.pairs_excluded_restraint = [0, 1, 0.1, 2]

        # Geometric complementarity restraint:
        # the parameters are the ids of the 2 components, the name of the restraint
        # maximum distance - if 2 components are apart by more than this distance,
        #   the restraint resturns the value 0
        # max_penetration. It is maximum distance that 2 rigid bodies can interpenetrate
        #           before the restraint returns a infinte value. For values of
        #           the interpenetration lower than maximum penetration, the
        #           restraint returns increasing positive values.
        # The value of the restraint is negative only when the subunits show
        # some surface complementarity.
        # Note: There is a known issue with this restraint. Sometimes the program will crash
        # for no apparent reason, and seems to be structure specific. One of the first
        # to do in this case is to remove this restraints.
        # the weight parameter is the weight for the restraint.
        # component1, component2, max distance,max penetration,weight.
        self.complementarity_restraints = [
            ["3sfdB", "3sfdA", "cB_A", 30, 15, 0.0001],
            ["3sfdB",
             "3sfdC",
             "cB_C",
             30,
             15,
             0.0001],
            ["3sfdB",
             "3sfdD",
             "cB_D",
             30,
             15,
             0.0001],
        ]

        # Em2DRestraint. Restraint measuring the similarity between a model and the
        # EM class averages. The parameters are:
        # the name of the restraint
        # the selection file containing the names of the images to use as
        # restraints.
        # The pixel size of the images
        # The resolution used to generate the projections of the model
        # The number of images used for the coarse search part of the em2d calculation.
        # The weight for the restraint
        # and the maximum score tolerated for the restraint. If the value for the
        # maximum score. 1000 is used here because the maximum value for the em2d score
        # is 1 and the weight of the restraint is 1000.
        self.em2d_restraints = [["em2d", "em_images/images.sel", 1.5, 5, 20, 1000, 1000]
                                ]

        # Cross-linking restraints. The parameters
        # the ids, chains and residue number of the subunits implied in the restraint
        # The maximum distance between residues tolerated. If the distance
        # is lower, the restraint has 0 value.
        # For the value of the stddev parameter, see the help for the
        # self.pair_score_restraints.
        # You can add a last parameter to the list, the maximum value of the scoring function
        # that you want to accept. It is only used during the DOMINO optimization
        # component1,chain1,residue1, component2, chain2, residue2, distance,weight, stddev, max_score
        self.xlink_restraints = [
            ["3sfdB", "B", 23, "3sfdA", "A", 456, 30, 100, 2],
            ["3sfdB",
             "B",
             241,
             "3sfdC",
             "C",
             112,
             30,
             100,
             2],
            ["3sfdB",
             "B",
             205,
             "3sfdD",
             "D",
             37,
             30,
             100,
             2],
            ["3sfdB",
             "B",
             177,
             "3sfdD",
             "D",
             99,
             30,
             100,
             2],
            ["3sfdC",
             "C",
             95,
             "3sfdD",
             "D",
             132,
             30,
             100,
             2],
            ["3sfdC",
             "C",
             9,
             "3sfdD",
             "D",
             37,
             30,
             100,
             2],
            ["3sfdC",
             "C",
             78,
             "3sfdD",
             "D",
             128,
             30,
             100,
             2],
        ]
        # set the value to False of this parameter if you don't have the HEXDOCK docking program.
        # self.have_hexdock = False

        #
        self.dock_transforms = [
            ["3sfdB", "3sfdA", "relative_positions_3sfdB-3sfdA.txt"],
            ["3sfdB", "3sfdC", "relative_positions_3sfdB-3sfdC.txt"],
            ["3sfdB", "3sfdD", "relative_positions_3sfdB-3sfdD.txt"],
            ["3sfdD", "3sfdC", "relative_positions_3sfdD-3sfdC.txt"],
        ]

        self.sampling_positions = DominoSamplingPositions()

        self.monte_carlo = MonteCarloParams()
        self.domino_params = DominoParams()

        self.test_opts = TestParameters()
        # benchmark
        self.benchmark = Benchmark()

        # results
        # the maximum number of solutions to produce
        self.n_solutions = 2000
        self.orderby = "em2d"



#!/usr/bin/env python

"""Build Files.cmake listing source files for each src, test, examples,
   benchmark directory in the subtree."""

import sys
import os
import os.path

sys.path.append(os.path.split(sys.argv[0])[0])
import python_tools


def _get_files(ds, suffix):
    ret = []
    for (dirpath, dirnames, filenames) in os.walk(ds):
        reldirpath = dirpath[len(ds) + 1:]
        for f in filenames:
            if f.endswith(suffix) and not f.startswith("_"):
                joined = os.path.join(reldirpath, f)
                ret.append(joined.replace("\\", "/"))
    ret.sort()
    return ret


def make_files(d):
    output = os.path.join(d, "Files.cmake")
    cppfiles = _get_files(d, ".cpp")
    cudafiles = _get_files(d, ".cu")
    pyfiles = _get_files(d, ".py")
    jsonfiles = _get_files(d, ".json")
    out = ["set(pyfiles \"%s\")" % ";".join(pyfiles),
           "set(cppfiles \"%s\")" % ";".join(cppfiles),
           "set(cudafiles \"%s\")" % ";".join(cudafiles)]
    if len(jsonfiles) > 0:
        out.append("set(jsonfiles \"%s\")" % ";".join(jsonfiles))
    python_tools.rewrite(output, "\n".join(out) + "\n")


def search_start(d):
    for cd in [o for o in os.listdir(d) if os.path.isdir(os.path.join(d, o))]:
        if cd.startswith("."):
            continue
        cp = os.path.join(d, cd)
        if cd in ["bin", "src", "test", "examples", "benchmark", "utility"]:
            make_files(cp)
        elif not cp.endswith('gitflow'):
            search_start(cp)


if __name__ == '__main__':
    search_start(".")



#!/usr/bin/env python

__doc__ = "Generate proteomics info from anchor graph and fits."

# read the anchors
# read the top fit for each protein, and assign the anchors
# add EV accordinly

import IMP.multifit
from IMP import ArgumentParser


def parse_args():
    desc = """
Generate a proteomics file automatically from the anchor graph and fitting
results. No interaction data is entered here, but the file can be modified
manually afterwards to add additional proteomics information.
"""
    p = ArgumentParser(description=desc)
    p.add_argument("assembly_file", help="assembly file name")
    p.add_argument("anchors_file", help="anchors file name")
    p.add_argument("proteomics_file", help="output proteomics file name")
    return p.parse_args()


def run(asmb_fn, anchors_fn, proteomics_fn):
    asmb = IMP.multifit.read_settings(asmb_fn)
    asmb.set_was_used(True)
    ad = IMP.multifit.read_anchors_data(anchors_fn)

    # read molecules
    mdl = IMP.Model()
    mhs = []
    centroids = []
    for i in range(asmb.get_number_of_component_headers()):
        fn = asmb.get_component_header(i).get_filename()
        mhs.append(IMP.atom.read_pdb(fn, mdl))
        centroids.append(IMP.core.get_centroid(IMP.core.get_leaves(mhs[i])))
    # matched anchors
    match = []
    for pt in ad.points_:
        min_len = 999999
        min_ind = 0
        for j in range(len(mhs)):
            dist = IMP.algebra.get_squared_distance(pt, centroids[j])
            if dist < min_len:
                min_len = dist
                min_ind = j
        match.append(min_ind)
    # now add all the EV
    ev_pairs = []
    for ind1, ind2 in ad.edges_:
        ev_pairs.append([match[ind1], match[ind2]])
    outf = open(proteomics_fn, "w")
    outf.write("|proteins|\n")
    for i, mh in enumerate(mhs):
        numres = len(IMP.atom.get_by_type(mh, IMP.atom.RESIDUE_TYPE))
        outf.write("|%s|1|%d|nn|nn|\n"
                   % (asmb.get_component_header(i).get_name(), numres))
    outf.write("|interactions|\n")
    outf.write("|residue-xlink|\n")
    outf.write("|ev-pairs|\n")
    pairs_map = {}
    for evp in ev_pairs:
        if evp[0] != evp[1]:
            sortpair = (min(*evp), max(*evp))
            if sortpair not in pairs_map:
                name0 = asmb.get_component_header(evp[0]).get_name()
                name1 = asmb.get_component_header(evp[1]).get_name()
                outf.write("|%s|%s|\n" % (name0, name1))
                pairs_map[sortpair] = 1
    outf.close()


def main():
    args = parse_args()
    run(args.assembly_file, args.anchors_file, args.proteomics_file)

if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
Set up a module by
- checking that all required modules and dependencies are found
- creating the config header and .cpp and version check .py
- linking .py files from the bin and benchmarks directories into the build dir

If the module cannot be configured, the script exits with an error.
"""

import sys
from optparse import OptionParser
import os.path
import tools

TOPDIR = os.path.abspath(os.path.dirname(__file__))

parser = OptionParser()
parser.add_option("-n", "--name",
                  dest="name", help="The name of the module.")
parser.add_option("-s", "--source",
                  dest="source", help="The root for IMP source.")
parser.add_option("-d", "--datapath",
                  dest="datapath", default="", help="An extra IMP datapath.")


def make_cpp(options):
    dir = os.path.join("src")
    file = os.path.join(dir, "%s_config.cpp" % options.name)
    cpp_template = tools.CPPFileGenerator(
        os.path.join(TOPDIR, "config_templates", "src.cpp"))
    try:
        os.makedirs(dir)
    except OSError:
        # exists
        pass
    data = {}
    if options.name == 'kernel':
        data["filename"] = "IMP/%s_config.h" % options.name
    else:
        data["filename"] = "IMP/%s/%s_config.h" % (options.name, options.name)
    data["cppprefix"] = "IMP%s" % options.name.upper().replace("_", "")
    data["name"] = options.name
    data["version"] = tools.get_module_version(options.name, options.source)
    cpp_template.write(file, data)


def make_version_check(options):
    dir = os.path.join("lib", "IMP",
                       "" if options.name == 'kernel' else options.name)
    tools.mkdir(dir, clean=False)
    version = tools.get_module_version(options.name, options.source)
    outf = os.path.join(dir, "_version_check.py")
    template = """def check_version(myversion):
  def _check_one(name, expected, found):
      if expected != found:
          raise RuntimeError(
              "Expected version " + expected + " but got " + found
              + " when loading module " + name
              + ". Please make sure IMP is properly built and installed "
              + "and that matching python and C++ libraries are used.")
  version = '%s'
  _check_one('%s', version, myversion)
  """
    g = tools.PythonFileGenerator()
    g.write(outf, template % (version, options.name))


def main():
    (options, args) = parser.parse_args()
    make_cpp(options)
    make_version_check(options)
    sys.exit(0)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

from __future__ import print_function
import sys
import os
import re
import shutil


class LogStep:

    def __init__(self, stepno, statline, header):
        self.stepno = stepno
        self.stats = statline
        self.header = header
        self.dumps = {}
        self.trajs = {}

    def add(self, ftype, category, *data):
        if ftype == 'dump':
            self.dumps[category] = data[0]
        elif ftype == 'traj':
            self.trajs[category] = {'ftype': data[0],
                                    'fullpath': data[1],
                                    'stepno': data[2],
                                    'tail': data[3]}
        else:
            raise ValueError("unknown file type")

    def get_stats_header(self):
        return self.header

    def get_stats(self):
        return self.stats

    def get_dumps(self):
        return self.dumps

    def get_trajs(self):
        return self.trajs


class LogHolder:

    """Manages information on a given simulation.
    Assumes the existence of a _stats.txt file, and handles more files if
    available.
    folder : the folder which contains _stats.txt
    prefix : the stats file is supposed to be prefix+'_stats.txt'
    """

    def __init__(self, folder, prefix):
        self.folder = folder
        self.prefix = prefix
        # verify that stats file exists
        self.stats_file = os.path.join(folder, prefix + 'stats.txt')
        if not os.path.isfile(self.stats_file):
            raise ValueError('cannot find stats file %s' % self.stats_file)
        # scan for other files
        files = {}
        for fl in os.listdir(folder):
            # get variable part of file, exclude stats file
            match = re.match(prefix + r'(.*)', fl)
            if match is None:
                continue
            tail = match.group(1)
            if tail == 'stats.txt':
                continue
            # get file category and add to files dict
            category = tail.split('_')[0]
            if not category in files:
                files[category] = []
            files[category].append(tail)
        # see if there are multiple files in the same category, and store them
        self.dumpfiles = {}
        self.trajfiles = {}
        for cat, fnames in files.items():
            if len(fnames) > 1 \
                    or os.path.splitext(fnames[0].split('_')[-1])[0].isdigit():
                # there are multiple files, no need to understand their content
                if not cat in self.dumpfiles:
                    self.dumpfiles[cat] = []
                for fname in fnames:
                    # parse tail and find index number
                    indexno = int(os.path.splitext(fname.split('_')[1])[0])
                    self.dumpfiles[cat].append((indexno, fname))
                self.dumpfiles[cat] = dict(self.dumpfiles[cat])
                # make sure there are no duplicate index numbers
                if len(self.dumpfiles[cat]) != \
                        len(set(self.dumpfiles[cat].keys())):
                    raise ValueError("found duplicates in %s %s %s"
                                     % (folder, prefix, fname))
            else:
                # this is a trajectory, need to be able to parse it
                fname = fnames[0]
                ext = os.path.splitext(fname)[1]
                if ext.startswith('.rmf'):
                    self.trajfiles[cat] = (ext[1:], fname)
                else:
                    raise ValueError("Unknown extension: %s in file %s"
                                     % (ext, fname))

    def get_stats_header(self):
        if not hasattr(self, 'stats_handle'):
            self.stats_handle = open(self.stats_file)
            # read the file and guess the number of lines
            # for now, be compatible with only one line
            self.stats_handle.readline()
            self.stats_first_line = self.stats_handle.readline()
            if self.stats_first_line.startswith('#'):
                raise ValueError('stats file must be 1-line only')
            self.stats_handle = open(self.stats_file)
            self.stats_header = self.stats_handle.readline()
        return self.stats_header

    def get_first_stats_line(self):
        # make sure file is open, skip header
        self.get_stats_header()
        return self.stats_first_line

    def _get_next_stats(self):
        # make sure file is open, skip header
        self.get_stats_header()
        for line in self.stats_handle:
            yield line

    def items(self):
        """iterate over all time steps"""
        # open stats file, store header and loop over stats file
        for stat in self._get_next_stats():
            # extract step number and create LogStep
            stepno = int(stat.split()[1])
            step = LogStep(stepno, stat, self.get_stats_header())
            # get other files' entries at this step if available
            for cat, df in self.dumpfiles.items():
                if stepno in df:
                    fullpath = os.path.join(self.folder,
                                            self.prefix + df[stepno])
                    step.add('dump', cat, fullpath)
            for cat, tf in self.trajfiles.items():
                fullpath = os.path.join(self.folder,
                                        self.prefix + tf[1])
                step.add('traj', cat, tf[0], fullpath, stepno, tf[1])
            # yield a LogStep containing these entries
            yield step


class Demuxer:

    """uses column to demux a replica trajectory. Assumes column points to a
    float or integer type, which is allowed to change over time. Attribution is
    based on order of float params. State 0 will be lowest param etc. Use
    reverse=True to start with highest.
    """

    def __init__(self, logs, outfolder, column, reverse=False):
        self.logs = logs
        self.reverse = reverse
        self.column = column
        self.outfolder = outfolder
        self.stat_handles = {}
        self.traj_handles_in = {}
        self.traj_handles_out = {}
        self.folders = {}
        # create needed folders
        if not os.path.isdir(outfolder):
            os.mkdir(outfolder)
        for l in range(len(self.logs)):
            fname = os.path.join(outfolder, 'p%d' % l)
            if not os.path.isdir(fname):
                os.mkdir(fname)
            self.folders[l] = fname
        # make sure every log has the same header
        h0 = self.logs[0].get_stats_header()
        for log in self.logs[1:]:
            if h0 != log.get_stats_header():
                raise ValueError("headers must be identical!")
        # get column number from header
        tokens = [idx for idx, i in enumerate(h0.split()) if self.column in i]
        if len(tokens) == 0:
            raise ValueError("column %d not found in this header\n%s"
                             % (column, h0))
        elif len(tokens) > 1:
            raise ValueError("column %d found multiple times!\n%s"
                             % (column, h0))
        self.colno = tokens[0]

    def get_param(self, statline):
        return float(statline.split()[self.colno])

    def _write_step_stats(self, stateno, lstep):
        # check if stats file is open
        if stateno not in self.stat_handles:
            self.stat_handles[stateno] = open(
                os.path.join(self.folders[stateno],
                             str(stateno) + '_stats.txt'), 'w')
            self.stat_handles[stateno].write(lstep.get_stats_header())
        # write stats
        self.stat_handles[stateno].write(lstep.get_stats())

    def _write_step_dump(self, stateno, lstep):
        for cat, fname in lstep.get_dumps().items():
            shutil.copyfile(fname, os.path.join(self.folders[stateno],
                                                str(stateno) + '_' + cat + fname.split(cat)[1]))

    def _write_traj_rmf(self, infile, instep, outfile, stateno, cat):
        import RMF
        # make sure infile is open
        if infile not in self.traj_handles_in:
            src = RMF.open_rmf_file_read_only(infile)
            self.traj_handles_in[infile] = src
        src = self.traj_handles_in[infile]
        # make sure outfile is open
        if outfile not in self.traj_handles_out:
            dest = RMF.create_rmf_file(outfile)
            self.traj_handles_out[outfile] = dest
            RMF.clone_file_info(src, dest)
            RMF.clone_hierarchy(src, dest)
            RMF.clone_static_frame(src, dest)
        dest = self.traj_handles_out[outfile]
        # clone frame
        frameid = src.get_frames()[instep - 1]
        src.set_current_frame(frameid)
        dest.add_frame(src.get_name(frameid), src.get_type(frameid))
        RMF.clone_loaded_frame(src, dest)

    def _write_step_traj(self, stateno, lstep):
        # loop over categories
        for cat, data in lstep.get_trajs().items():
            destfile = os.path.join(self.outfolder, 'p' + str(stateno),
                                    str(stateno) + '_' + data['tail'])
            if data['ftype'].startswith('rmf'):
                self._write_traj_rmf(data['fullpath'], data['stepno'],
                                     destfile, stateno, cat)
            else:
                raise ValueError("unknown trajectory file type")

    def _write_step(self, stateno, lstep):
        self._write_step_stats(stateno, lstep)
        self._write_step_dump(stateno, lstep)
        self._write_step_traj(stateno, lstep)

    def write(self):
        # loop over time steps
        log_iterators = [list(l.items()) for l in self.logs]
        print("Demuxing", len(log_iterators), "replicas")
        for idx, steps in enumerate(zip(*log_iterators)):
            if idx % 10 == 0 and idx > 0:
                print("step", idx, '\r', end=' ')
                sys.stdout.flush()
            # assign state numbers to these logs
            params = [(self.get_param(i.get_stats()), i) for i in steps]
            params.sort(reverse=self.reverse)
            # write them
            for i in range(len(params)):
                self._write_step(i, params[i][1])
        print("Done")


def get_prefix(folder):
    rval = [re.match(r'(.*_)stats.txt', f) for f in os.listdir(folder)]
    rval = [i for i in rval if i]
    if len(rval) != 1:
        raise ValueError("stats file not unique, found %d" % len(rval))
    return rval[0].group(1)

if __name__ == '__main__':
    if len(sys.argv) == 1 or len(sys.argv) > 4:
        sys.exit("""demux_trajs.py column [infolder [outfolder]]
        expects r?? folders in infolder and will write p?? folders in outfolder.
        infolder must contain a _stats.txt file which will contain a header.
        column must be a substring matching to one of the columns in the
        _stats.txt files. It will typically be a temperature, or a state number.
        That column will be used for demuxing. Folders are optional and will be
        taken as ./ if not indicated.
        """)
    column = sys.argv[1]
    if len(sys.argv) == 3:
        infolder = sys.argv[2]
        outfolder = './'
    elif len(sys.argv) == 4:
        infolder = sys.argv[2]
        outfolder = sys.argv[3]
    else:
        infolder = './'
        outfolder = './'
    # loop over infolder and read stats files
    folders = [os.path.join(infolder, f)
               for f in os.listdir(infolder) if re.match(r'r\d+', f)]
    replica_logs = [LogHolder(f, prefix)
                    for f, prefix in zip(folders, map(get_prefix, folders))]
    demux = Demuxer(replica_logs, outfolder, column, reverse=True)
    demux.write()



import IMP
import IMP.core
import IMP.algebra
import itertools
import math
import random
import numpy as np
import scipy.spatial
import networkx as nx
from IMP.pmi.tools import get_restraint_set


class MoverScheme(object):
    def __init__(self, length):
        self.length = length
        self.mover_scheme = None
        self.score_scheme = {}
        self.score_initial_weights = {}
        self.position = 0
        self.numbermc = 0

    def add_mover_scheme(self, mover_scheme):
        if len(mover_scheme) != self.length:
            raise ("Error")
        else:
            mc_scheme = []
            for mvs in mover_scheme:
                smv = IMP.core.SerialMover(mvs)
                mc = IMP.core.MonteCarlo(mvs[0].get_model())
                mc.set_scoring_function(get_restraint_set(mvs[0].get_model()))
                mc.set_return_best(False)
                mc.set_kt(1.0)
                mc.add_mover(smv)
                mc_scheme.append(mc)
                mc.set_name(str(self.numbermc))
                self.numbermc += 1
            self.mover_scheme = mc_scheme

    def add_montecarlo_temperatures(self, temps):
        self.temps = temps

    def add_montecarlo_steps(self, steps):
        self.steps = steps

    def add_score_scheme(self, score, weigth_scheme):
        if len(weigth_scheme) != self.length:
            raise ("Error")
        else:
            self.score_scheme[score] = weigth_scheme
        self.score_initial_weights[score] = score.get_weight()

    def get_length(self):
        return self.length

    def get_position(self, position):
        movers = self.mover_scheme[position]
        score_tmp_dict = {}
        for score in self.score_scheme:
            score_tmp_dict[score] = self.score_scheme[score][position]
        return movers, score_tmp_dict

    def get_and_update(self):
        mc, score_tmp_dict = self.get_position(self.position)
        temp = self.temps[self.position]
        mc.set_kt(temp)
        steps = self.steps[self.position]
        self.position += 1
        if self.position > self.length - 1: self.position = 0
        return mc, score_tmp_dict, steps


class HybridMonteCarloMover(IMP.core.MonteCarloMover):
    def __init__(self, m, hier, MoverScheme):
        IMP.core.MonteCarloMover.__init__(
            self,
            m,
            "HybridMonteCarloMover")

        self.mols = IMP.pmi.tools.get_molecules(hier)
        self.refframes = {}
        self.xyzs = {}
        self.ps = []
        self.m = m
        self.rejects = []
        self.lenrejects = 20

        self.rbs, self.beads = IMP.pmi.tools.get_rbs_and_beads([hier])
        for rb in self.rbs:
            self.ps.append(rb.get_particle())
        for bead in self.beads:
            self.ps.append(bead.get_particle())

        self.indexes = [p.get_index() for p in self.ps]

        self.do_store_current_coordinates()

        self.mover_scheme = MoverScheme

    def do_store_current_coordinates(self):
        for rb in self.rbs:
            self.refframes[rb] = rb.get_reference_frame()
        for bead in self.beads:
            self.xyzs[bead] = IMP.core.XYZ(bead).get_coordinates()

    def do_propose(self):

        self.rejects.append(0)
        if len(self.rejects) > self.lenrejects:
            self.rejects.pop(0)
        print
        self.rejects, float(sum(self.rejects)) / len(self.rejects)

        self.do_store_current_coordinates()

        rs = get_restraint_set(self.m)
        # print "Initial",rs.evaluate(False)

        for i in range(self.mover_scheme.get_length()):
            mc, score_tmp_dict, steps = self.mover_scheme.get_and_update()

            mc.reset_statistics()

            for score in score_tmp_dict:
                score.set_weight(score_tmp_dict[score])

            mc.optimize(steps)

            print
            mc.get_name(), float(mc.get_number_of_accepted_steps()) / mc.get_number_of_proposed_steps()
        # print i,steps,rs.evaluate(False)

        # print "Final",rs.evaluate(False)

        return IMP.core.MonteCarloMoverResult(self.indexes, 1.0)

    def do_reject(self):
        print
        "REJECT"
        self.rejects[-1] = 1
        for rb in self.refframes:
            rb.set_reference_frame(self.refframes[rb])
        for bead in self.xyzs:
            IMP.core.XYZ(bead).set_coordinates(self.xyzs[bead])

    def do_get_inputs(self):
        return self.ps


class PCAMover(IMP.core.MonteCarloMover):
    def __init__(self, m, hier, resolution=10, depth=4):

        IMP.core.MonteCarloMover.__init__(
            self,
            m,
            "PCAMover")
        self.depth = depth
        self.propose = 0
        self.reject = 0
        self.rejects = []
        self.lenrejects = 20
        self.mols = IMP.pmi.tools.get_molecules(hier)
        self.resolution = resolution
        self.rbsd = {}
        self.beadsd = {}
        self.refframes = {}
        self.xyzs = {}
        self.ps = []
        self.particle_blocks = []
        for mol in self.mols:
            rbs, beads = IMP.pmi.tools.get_rbs_and_beads([mol])
            self.rbsd[mol] = rbs
            self.beadsd[mol] = beads
            for rb in self.rbsd[mol]:
                self.ps.append(rb.get_particle())
            for bead in self.beadsd[mol]:
                self.ps.append(bead.get_particle())
            ps = IMP.atom.Selection(mol, resolution=self.resolution).get_selected_particles()
            self.particle_blocks.append(ps)

        self.do_store_current_coordinates()

    def do_store_current_coordinates(self):
        for mol in self.mols:
            for rb in self.rbsd[mol]:
                self.refframes[rb] = rb.get_reference_frame()
            for bead in self.beadsd[mol]:
                self.xyzs[bead] = IMP.core.XYZ(bead).get_coordinates()

    def get_full_graph(self):
        '''
        get the full graph of distances between every particle pair
        '''

        pdist_array = np.array(
            IMP.pmi.get_list_of_bipartite_minimum_sphere_distance(self.particle_blocks))
        pdist_mat = scipy.spatial.distance.squareform(pdist_array)
        pdist_mat[pdist_mat <= 10] = 1
        pdist_mat[pdist_mat > 10] = 0
        graph = nx.Graph(pdist_mat)
        return graph

    def get_list_of_interacting_molecules(self, succ_dict, pointer):
        if pointer in succ_dict:
            paths = [path for p in succ_dict[pointer] for path in self.get_list_of_interacting_molecules(succ_dict, p)]
            ret = [[pointer] + path for path in paths]
            # print pointer,paths,ret
            return ret
        else:
            return [[pointer]]

    def get_minimum_spanning_tree(self):
        """
        return the minimum spanning tree
        """
        graph = self.get_full_graph()
        graph = nx.minimum_spanning_tree(graph)
        return graph

    def get_sublists(self, lst, size):
        result = [sublist for sublist in
                  (lst[x:x + size] for x in range(len(lst) - size + 1))]
        return result

    def get_list_of_connected_mols(self, depth):
        gr = self.get_minimum_spanning_tree()
        all_groups = set()
        if len(gr.edges()) > 0:

            for node in gr.nodes():
                succ_dict = nx.dfs_successors(gr, node)
                paths = [path for p in succ_dict[node] for path in self.get_list_of_interacting_molecules(succ_dict, p)]
                ret = [[node] + path for path in paths]
                for path in ret:
                    for s in range(2, min([depth + 1, len(path) + 1, len(self.mols)])):
                        subls = self.get_sublists(path, s)
                        for subl in subls: all_groups.add(frozenset(subl))
        return all_groups
        # succ=nx.bfs_successors(gr,0)

    # succmol=dict([(self.mols[i].get_name()+str(IMP.atom.Copy(self.mols[i]).get_copy_index()),[self.mols[k].get_name()+str(IMP.atom.Copy(self.mols[i]).get_copy_index()) for k in ks]) for i,ks in succ.iteritems()])
    # print(succmol)

    def do_compute_pcas(self):
        self.singletons = {}
        for i, ps in enumerate(self.particle_blocks):
            mol = self.mols[i]
            pca = IMP.algebra.get_principal_components([IMP.core.XYZ(p).get_coordinates() for p in ps])
            ## MB normalize
            pcan = IMP.bayesianem.NormalizePCA(pca, ps)
            self.singletons[(mol,)] = pcan

        if self.depth > 1:
            all_groups = self.get_list_of_connected_mols(self.depth)

            for group in all_groups:
                mols = [self.mols[n] for n in group]
                ps = IMP.atom.Selection(mols, resolution=self.resolution).get_selected_particles()
                pca = IMP.algebra.get_principal_components([IMP.core.XYZ(p).get_coordinates() for p in ps])
                pcan = IMP.bayesianem.NormalizePCA(pca, ps)
                self.singletons[tuple(mols)] = pcan

    def do_get_pairs(self, temp=20):
        sk = self.singletons.keys()

        pairs = []
        for s1, s2 in itertools.combinations(sk, 2):
            rms = IMP.algebra.get_distance(self.singletons[s1].get_principal_values(),
                                           self.singletons[s2].get_principal_values())
            pairs.append(((s1, s2), math.exp(-rms / temp)))

        return pairs

    def do_compute_transforms(self, pair):
        amols = pair[0]
        bmols = pair[1]

        arbs = []
        abeads = []
        for mol in amols:
            arbs += self.rbsd[mol]
            abeads += self.beadsd[mol]
        apca = self.singletons[amols]

        brbs = []
        bbeads = []
        for mol in bmols:
            brbs += self.rbsd[mol]
            bbeads += self.beadsd[mol]
        bpca = self.singletons[bmols]

        # transatob=IMP.algebra.get_alignments_from_first_to_second(apca,bpca)
        # MB get single transformation
        transatob = IMP.bayesianem.PCAalign(apca, bpca)[0]
        transbtoa = transatob.get_inverse()

        return (transatob, transbtoa, arbs, abeads, brbs, bbeads)

    def weighted_choice(self, choices):
        total = sum(w for c, w in choices)
        r = random.uniform(0, total)
        upto = 0
        for c, w in choices:
            if upto + w >= r:
                return c
            upto += w
        assert False, "Shouldn't get here"

    def do_propose(self):
        self.rejects.append(0)
        if len(self.rejects) > self.lenrejects:
            self.rejects.pop(0)

        self.propose += 1
        self.do_store_current_coordinates()
        self.do_compute_pcas()
        pairs = self.do_get_pairs()
        pair = self.weighted_choice(pairs)
        # print pair,self.reject,self.propose,float(self.propose-self.reject)/self.propose,float(sum(self.rejects))/len(self.rejects)
        # print self.rejects
        t = self.do_compute_transforms(pair)
        # (ta2b,tb2a)=random.choice(zip(t[0],t[1]))
        # MB no need to randomly select
        (ta2b, tb2a) = (t[0], t[1])

        moved_indexes = []
        for arb in t[2]:
            IMP.core.transform(arb, ta2b)
            moved_indexes.append(arb.get_particle_index())
        for abead in t[3]:
            IMP.core.transform(IMP.core.XYZ(abead), ta2b)
            moved_indexes.append(arb.get_particle_index())
        for brb in t[4]:
            IMP.core.transform(brb, tb2a)
        for bbead in t[5]:
            IMP.core.transform(IMP.core.XYZ(bbead), tb2a)
        return IMP.core.MonteCarloMoverResult(moved_indexes, 1.0)

    def do_reject(self):
        self.rejects[-1] = 1

        self.reject += 1
        for rb in self.refframes:
            rb.set_reference_frame(self.refframes[rb])
        for bead in self.xyzs:
            IMP.core.XYZ(bead).set_coordinates(self.xyzs[bead])

    def do_get_inputs(self):
        return self.ps



## \example atom/brownian_statistics.py
# This example prints out various statistics about a prospective Brownian
# dynamics simulation. You can use the statistics to determine if the time
# step is likely to be sufficiently short given the forces and particle
# sizes involved.

from __future__ import print_function
import IMP.atom
import sys

IMP.setup_from_argv(sys.argv, "brownian statistics")

# fs
time_step = 1000
# angstrom
minimum_particle_radius = 10
# kCal/mol/A
maximum_spring_constant = 1

maximum_diffusion_coefficient =\
    IMP.atom.get_einstein_diffusion_coefficient(minimum_particle_radius)
expected_delta = IMP.atom.get_diffusion_length(maximum_diffusion_coefficient,
                                               time_step)

expected_rotational_delta =\
    IMP.atom.get_diffusion_angle(maximum_diffusion_coefficient,
                                 time_step) * minimum_particle_radius

expected_spring_diffusion_length =\
    IMP.atom.get_diffusion_length(maximum_diffusion_coefficient,
                                  .5 * maximum_spring_constant *
                                  4 * expected_delta ** 2, time_step)

print("with a time step of", time_step, "fs")
print("an object of radius", minimum_particle_radius, "A will move",\
    expected_delta, "A and a point on its surface will move",\
    expected_rotational_delta, "A more")
print("the motion from fluctuations in the spring compression will be",\
    expected_spring_diffusion_length, "A")
print("and a compression of 10% of the radius will induce a motion of",\
    IMP.atom.get_diffusion_length(maximum_diffusion_coefficient,
                         .5 * maximum_spring_constant *
                         (.1 * minimum_particle_radius) ** 2,
                         time_step), "A")



#!/usr/bin/env python
from __future__ import print_function
import os
from IMP import ArgumentParser


__doc__ = "List/extract good-scoring models from a set of sampling runs."""


def parse_args():
    parser = ArgumentParser(
        description="List and extract good-scoring models from a set of "
        "sampling runs. Example of usage: "
        "%(prog)s -rd <run_directory_for_sampling> -rp <run_prefix> -sl "
        "ExcludedVolumeSphere_None GaussianEMRestraint_None "
        "-pl CrossLinkingMassSpectrometryDataScore|"
        "XLDSS CrossLinkingMassSpectrometryDataScore|XLEDC -agl -9999999.0 "
        "-99999.0 -aul 99999999.0 999999.0 -mlt 0 0 -mut 0 0. Flag -h "
        "for more details.")

    parser.add_argument("-rd", "--run_directory", dest="run_dir",
                        help="directory in which sampling results are stored",
                        required=True)

    parser.add_argument("-rp", "--run_prefix", dest="run_prefix",
                        help="prefix of runs", required=True)

    parser.add_argument("-sl", "--selection_keywords_list", nargs='+',
                        type=str, dest="selection_keywords_list",
                        help="list of stat file keywords corresponding to "
                             "selection criteria")
    parser.add_argument("-pl", "--printing_keywords_list", nargs='+',
                        type=str, dest="printing_keywords_list",
                        help="list of stat file keywords whose values are "
                             "printed out for selected models")

    # thresholds only apply to selection keywords
    parser.add_argument("-alt", "--aggregate_lower_thresholds", nargs='+',
                        type=float, dest="aggregate_lower_thresholds",
                        help="aggregate lower thresholds")
    parser.add_argument("-aut", "--aggregate_upper_thresholds", nargs='+',
                        type=float, dest="aggregate_upper_thresholds",
                        help="aggregate upper thresholds")
    parser.add_argument("-mlt", "--member_lower_thresholds", nargs='+',
                        type=float, dest="member_lower_thresholds",
                        help="member lower thresholds")
    parser.add_argument("-mut", "--member_upper_thresholds", nargs='+',
                        type=float, dest="member_upper_thresholds",
                        help="member upper thresholds")

    parser.add_argument("-e", "--extract", default=False, dest="extract",
                        action='store_true',
                        help="Type -e to extract all good scoring model "
                             "RMFs from the trajectory files")
    parser.add_argument("-sf", "--score_file", default="scores", type=str,
                        dest="score_file_prefix",
                        help="Score file prefix for samples A and B. "
                             "Default is %(default)r")
    result = parser.parse_args()

    return result


def select_good_scoring_models():
    from IMP.sampcon.good_scoring_model_selector \
            import GoodScoringModelSelector

    # process input
    arg = parse_args()

    gsms = GoodScoringModelSelector(arg.run_dir, arg.run_prefix)

    subsets = gsms.get_good_scoring_models(
        selection_keywords_list=arg.selection_keywords_list,
        printing_keywords_list=arg.printing_keywords_list,
        aggregate_lower_thresholds=arg.aggregate_lower_thresholds,
        aggregate_upper_thresholds=arg.aggregate_upper_thresholds,
        member_lower_thresholds=arg.member_lower_thresholds,
        member_upper_thresholds=arg.member_upper_thresholds,
        extract=arg.extract)
    return subsets, arg.extract


def create_score_files(subsets, field="Total_Score"):
    arg = parse_args()

    score_dir = os.path.join(arg.run_dir, "good_scoring_models")
    scoreA = open(os.path.join(score_dir, arg.score_file_prefix + "A.txt"),
                  "w")
    scoreB = open(os.path.join(score_dir, arg.score_file_prefix + "B.txt"),
                  "w")
    model_file = open(os.path.join(score_dir, "model_ids_scores.txt"), "r")

    print("Creating input files for Total_Score convergence test")

    for line_index, each_model_line in enumerate(model_file.readlines()):

        # Find index of the field we want to use for model score convergence
        if line_index == 0:
            field_headers = each_model_line.strip().split()

            try:
                ts_ix = field_headers.index(field)
                _ = field_headers.index("Run_id")
                model_ix = field_headers.index("Model_index")

            except ValueError:
                print("create_scores_file: model_ids_scores.txt file has "
                      "an incorrect format.")
                exit()

        else:
            fields = each_model_line.strip().split()

            score = fields[ts_ix]

            model = int(fields[model_ix])
            print(score, file=scoreA if model in subsets[0] else scoreB)

    scoreA.close()
    scoreB.close()


def main():
    subsets, extract = select_good_scoring_models()

    print("Filtered model scores ready")

    if extract:

        # Create Score Files
        create_score_files(subsets)

        print("Ready to calculate sampling precision with "
              "Master_Sampling_Exhaustiveness_Analysis.py")


if __name__ == "__main__":
    main()



"""@namespace IMP.EMageFit.utility
   Utility functions.
"""

import os
import IMP

def vararg_callback(option, opt_str, value, parser):
    """
        Snippet from Python website to process multiple values for
        an option with OptionParser
    """

    assert value is None
    value = []

    def floatable(str):
        try:
            float(str)
            return True
        except ValueError:
            return False

    for arg in parser.rargs:
        # stop on --foo like options
        if arg[:2] == "--" and len(arg) > 2:
            break
        # stop on -a, but not on -3 or -3.0
        if arg[:1] == "-" and len(arg) > 1 and not floatable(arg):
            break
        value.append(arg)

    del parser.rargs[:len(value)]
    setattr(parser.values, option.dest, value)

    return value


def get_experiment_params(fn_params):
    """
        Imports the configuration file
        @param fn_params configuration file
        @return Experiment Class with all the information from the config file
    """
    try:
        import importlib.machinery
        imp = None
    except ImportError:
        import imp
    name, ext = os.path.splitext(fn_params)
    if imp is None:
        foo = importlib.machinery.SourceFileLoader(name,
                                                   fn_params).load_module()
    else:
        foo = imp.load_source(name, fn_params)
    exp = foo.Experiment()
    # convert to absolute paths
    exp.fn_pdbs = [IMP.get_relative_path(fn_params, fn) for fn in exp.fn_pdbs]
    if hasattr(exp, "sampling_positions"):
        exp.sampling_positions.read = IMP.get_relative_path(
            fn_params, exp.sampling_positions.read)
    if hasattr(exp, "benchmark"):
        if hasattr(exp.benchmark, "fn_pdb_native"):
            exp.benchmark.fn_pdb_native = IMP.get_relative_path(
                fn_params, exp.benchmark.fn_pdb_native)
        if hasattr(exp.benchmark, "fn_pdbs_native"):
            fns = []
            for fn in exp.benchmark.fn_pdbs_native:
                fns.append(IMP.get_relative_path(fn_params, fn))
            exp.benchmark.fn_pdbs_native = fns

    if hasattr(exp, "dock_transforms"):
        for i in range(len(exp.dock_transforms)):
            exp.dock_transforms[i][2] = IMP.get_relative_path(
                fn_params, exp.dock_transforms[i][2])
    if hasattr(exp, "em2d_restraints"):
        for i in range(len(exp.em2d_restraints)):
            exp.em2d_restraints[i][1] = IMP.get_relative_path(
                fn_params, exp.em2d_restraints[i][1])
    return exp



from __future__ import print_function
import numpy
import math
import scipy.stats


def get_sample_identity(idfile_A, idfile_B):
    # whether a run belongs to run1 or run2
    sampleA_models = []
    sampleB_models = []

    with open(idfile_A, 'r') as f:
        for line in f:
            mod = line.split("/")[-1]
            sampleA_models.append(int(mod.strip("\n").split(" ")[1]))
    f.close()

    with open(idfile_B, 'r') as f:
        for line in f:
            mod = line.split("/")[-1]
            sampleB_models.append(int(mod.strip("\n").split(" ")[1]))
    return sampleA_models, sampleB_models


def get_cutoffs_list(distmat, gridSize):

    mindist = distmat.min()
    maxdist = distmat.max()

    print("Minimum and maximum pairwise model distances:", mindist, maxdist)
    cutoffs = numpy.arange(mindist+gridSize, maxdist, gridSize)
    return cutoffs


def precision_cluster(distmat, numModels, rmsd_cutoff):
    # STEP 2. Populate the neighbors ofa given model
    neighbors = []
    for count in range(numModels):
        neighbors.append([count])  # model is a neighbor of itself

    # set of i,j indices that pass
    inds = numpy.argwhere(distmat <= rmsd_cutoff)
    for x in inds:
        i = x[0]
        j = x[1]
        # Only count each contribution once
        if i > j:
            neighbors[i].append(j)
            neighbors[j].append(i)

    # STEP 3. Get the weightiest cluster, and iterate
    unclustered = []
    boolUnclustered = []
    for i in range(numModels):
        unclustered.append(i)
        boolUnclustered.append(True)

    cluster_members = []   # list of lists : one list per cluster
    cluster_centers = []

    while len(unclustered) > 0:
        # get cluster with maximum weight
        max_neighbors = 0
        currcenter = -1
        for eachu in unclustered:
            # if multiple clusters have same maxweight this tie is
            # broken arbitrarily!
            if len(neighbors[eachu]) > max_neighbors:
                max_neighbors = len(neighbors[eachu])
                currcenter = eachu

        # form a new cluster with u and its neighbors
        cluster_centers.append(currcenter)
        cluster_members.append([n for n in neighbors[currcenter]])

        # update neighbors
        for n in neighbors[currcenter]:
            # removes the neighbor from the pool
            unclustered.remove(n)  # first occurence of n is removed.
            boolUnclustered[n] = False  # clustered

        for n in neighbors[currcenter]:
            for unn in neighbors[n]:  # unclustered neighbor
                if not boolUnclustered[unn]:
                    continue
                neighbors[unn].remove(n)

    return cluster_centers, cluster_members


def get_contingency_table(num_clusters, cluster_members, all_models,
                          run1_models, run2_models):
    full_ctable = numpy.zeros((num_clusters, 2))

    for ic, cluster in enumerate(cluster_members):
        for member in cluster:
            model_index = all_models[member]

            if model_index in run1_models:
                # print("run1", model_index)
                full_ctable[ic][0] += 1.0
            elif model_index in run2_models:
                # print("run2", model_index)
                full_ctable[ic][1] += 1.0

    # now normalize by number of models in each run
    reduced_ctable = []
    retained_clusters = []

    for i in range(num_clusters):
        if full_ctable[i][0] <= 10.0 or full_ctable[i][1] <= 10.0:
            continue
        reduced_ctable.append([full_ctable[i][0], full_ctable[i][1]])
        retained_clusters.append(i)
    return numpy.array(reduced_ctable), retained_clusters


def test_sampling_convergence(contingency_table, total_num_models):
    if len(contingency_table) == 0:
        return 0.0, 1.0

    ct = numpy.transpose(contingency_table)
    [chisquare, pvalue, dof, expected] = scipy.stats.chi2_contingency(ct)
    if dof == 0.0:
        cramersv = 0.0
    else:
        cramersv = math.sqrt(chisquare/float(total_num_models))

    return pvalue, cramersv


def percent_ensemble_explained(ctable, total_num_models):
    if len(ctable) == 0:
        return 0.0
    percent_clustered = \
        float(numpy.sum(ctable, axis=0)[0] + numpy.sum(ctable, axis=0)[1]) \
        * 100.0 / float(total_num_models)
    return percent_clustered


def get_clusters(cutoffs_list, distmat_full, all_models, total_num_models,
                 run1_all_models, run2_all_models, sysname):
    # Do Clustering on a Grid
    pvals = []
    cvs = []
    percents = []
    with open("%s.ChiSquare_Grid_Stats.txt" % sysname, 'w+') as f1:
        for c in cutoffs_list:
            cluster_centers, cluster_members = precision_cluster(
                    distmat_full, total_num_models, c)
            ctable, retained_clusters = get_contingency_table(
                    len(cluster_centers), cluster_members, all_models,
                    run1_all_models, run2_all_models)
            pval, cramersv = test_sampling_convergence(ctable,
                                                       total_num_models)
            percent_explained = percent_ensemble_explained(ctable,
                                                           total_num_models)

            pvals.append(pval)
            cvs.append(cramersv)
            percents.append(percent_explained)

            print(c, pval, cramersv, percent_explained, file=f1)

    return pvals, cvs, percents


def get_sampling_precision(cutoffs_list, pvals, cvs, percents):
    sampling_precision = max(cutoffs_list)
    pval_converged = 0.0
    cramersv_converged = 1.0
    percent_converged = 0.0

    for i in range(len(cutoffs_list)):
        if percents[i] > 80.0:
            if pvals[i] > 0.05 or cvs[i] < 0.10:
                if sampling_precision > cutoffs_list[i]:
                    sampling_precision = cutoffs_list[i]
                    pval_converged = pvals[i]
                    cramersv_converged = cvs[i]
                    percent_converged = percents[i]
        else:
            sampling_precision = max(cutoffs_list)

    return (sampling_precision, pval_converged, cramersv_converged,
            percent_converged)



## \example example/range_restriction.py
# This example shows how to use the example singleton modifier to constrain the
# coordinates of a set of particles to remain within a box.

import IMP.example
import IMP.core
import IMP.container
import sys

# you can use this argument to shorten the test, if necessary
IMP.add_bool_flag("test", "Run a minimal test on the script")

# parse standard IMP flags
IMP.setup_from_argv(sys.argv, "A trivial example of an example.")


bb = IMP.algebra.BoundingBox3D(IMP.algebra.Vector3D(0, 0, 0),
                               IMP.algebra.Vector3D(10, 10, 10))

m = IMP.Model()
ps = IMP.core.create_xyzr_particles(m, 20, 1)
sc = IMP.container.ListSingletonContainer(m, ps)

# apply the range restriction modifier to each each particle in sc
ss = IMP.container.SingletonsConstraint(
    IMP.example.ExampleSingletonModifier(bb), None, sc)
m.add_score_state(ss)

# now optimize and things
# ...



#!/usr/bin/env python
from __future__ import print_function
import sys
import os

import IMP
import IMP.atom
import IMP.container
import IMP.isd
import IMP.isd.shared_functions
import IMP.isd.Statistics
import IMP.isd.Entry
sfo_common = IMP.isd.shared_functions.sfo_common
Statistics = IMP.isd.Statistics.Statistics
Entry = IMP.isd.Entry.Entry

import math

kB = (1.381 * 6.02214) / 4184.0


class sfo(sfo_common):

    "shared functions object, published on all nodes"

    def init_model(self, wd, seqfile, initpdb, noe, ff_temp=300.0):
        "loads pdb and restraints and creates particles and scales"
        # Create an IMP model and add a heavy atom-only protein from a PDB file
        self.init_model_base(wd)
        m = self._m
        #
        print("loading protein and force field")
        prot, ff, rsb, rs = self.init_model_charmm_protein_and_ff(initpdb,
                                                                  "top.lib", "par.lib",
                                                                  IMP.atom.NonWaterPDBSelector(
                                                                  ),
                                                                  IMP.isd.RepulsiveDistancePairScore(
                                                                      0, 1),
                                                                  ff_temp=300.0)
        self._rs = {}
        self._rs['phys'] = rs
        self._rs['phys_bonded'] = rsb
        self._p = {}
        self._p['prot'] = prot
        #
        print("NOE restraints")
        noe_rs, prior_rs, sigma, gamma = self.init_model_NOEs(prot,
                                                              seqfile, noe, name='NOE', prior_rs=None,
                                                              bounds_sigma=(
                                                                  1.0, 0.1, 100),
                                                              bounds_gamma=(1.0, 0.1, 100))
        self._rs['NOE'] = noe_rs
        self._rs['prior'] = prior_rs
        self._p['sigma'] = sigma
        self._p['gamma'] = gamma

    def init_simulation(self, lambda_temp=1.0, tau=500.0):
        "prepare md and mc sims"
        self.inv_temp = lambda_temp
        self.md_tau = tau
        self._md, self._os = self._setup_md(self._p['prot'],
                                            thermostat='berendsen',
                                            temperature=1 /
                                            (kB * lambda_temp),
                                            coupling=tau)
        rs_scales = [self._rs['NOE'], self._rs['prior']]
        self._mc_sigma, self._nm_sigma = \
            self.init_simulation_setup_nuisance_mc(
                self._p['sigma'], 1 / (kB * lambda_temp),
                mc_restraints=rs_scales, nm_stepsize=0.1)
        self._mc_gamma, self._nm_gamma = \
            self.init_simulation_setup_nuisance_mc(
                self._p['gamma'], 1 / (kB * lambda_temp),
                mc_restraints=rs_scales, nm_stepsize=0.1)

    def init_stats(self, prefix='p01', rate=1):
        # print statistics in files with prefix and at a certain rate (in gibbs
        # sampling steps).
        stat = Statistics(prefix, rate)
        self.stat = stat
        # general stuff to monitor: forcefield terms

        def get_ephys():
            return self._rs['phys_bonded'].evaluate(False) \
                + self._rs['phys'].evaluate(False)
        # the following line will raise a warning in debug mode, but it's ok.
        stat.add_entry('global', entry=Entry('E_phys', '%10f', get_ephys))
        stat.add_entry('global', entry=Entry('E_NOE', '%10f',
                                             self._rs['NOE'].evaluate, False))
        stat.add_entry('global', entry=Entry('E_prior', '%10f',
                                             self._rs['prior'].evaluate, False))
        # MD monitoring: target_temp, instant_temp, E_kinetic
        self.md_key = self.init_stats_add_md_category(stat)
        # MC monitoring: acceptance, stepsize, sigma
        self.sigma_mc_key = \
            self.init_stats_add_mc_category(stat, name='mc_sigma',
                                            coord='sigma')
        # MC monitoring: acceptance, stepsize, gamma
        self.gamma_mc_key = \
            self.init_stats_add_mc_category(stat, name='mc_gamma',
                                            coord='gamma')

    def do_md(self, nsteps):
        "perform md simulation on protein for nsteps"
        for i in IMP.atom.get_leaves(self._p['prot']):
            IMP.core.XYZR(i).set_coordinates_are_optimized(True)
        self._p['sigma'].set_is_optimized(IMP.FloatKey("scale"), False)
        self._p['gamma'].set_is_optimized(IMP.FloatKey("scale"), False)
        self._md.optimize(nsteps)
        self.do_md_protein_statistics(self.md_key, nsteps, self._md,
                                      temperature=1 / (kB * self.inv_temp),
                                      prot_coordinates=self.get_pdb(self._p['prot']))

    def do_mc(self, nsteps):
        """perform mc on scales for nsteps, updating stepsizes to target
        50% acceptance. Don't make nsteps too small (say << 50).
        """
        prot = self._p['prot']
        sigma = self._p['sigma']
        gamma = self._p['gamma']
        for i in IMP.atom.get_leaves(prot):
            IMP.core.XYZR(i).set_coordinates_are_optimized(False)
        sigma.set_is_optimized(IMP.FloatKey("scale"), True)
        gamma.set_is_optimized(IMP.FloatKey("scale"), False)
        self._mc_and_update_nm(nsteps, self._mc_sigma, self._nm_sigma,
                               self.sigma_mc_key, adjust_stepsize=True)
        self.stat.update(self.sigma_mc_key, 'sigma', sigma.get_scale())
        sigma.set_is_optimized(IMP.FloatKey("scale"), False)
        gamma.set_is_optimized(IMP.FloatKey("scale"), True)
        self._mc_and_update_nm(nsteps, self._mc_gamma, self._nm_gamma,
                               self.gamma_mc_key)
        self.stat.update(self.gamma_mc_key, 'gamma', gamma.get_scale())

    def write_stats(self):
        # increment global counter
        self.stat.increment_counter('global', 1)
        # write statistics if necessary
        self.stat.write_stats()

    def set_inv_temp(self, inv_temp):
        "sets inverse temperature of mc and md sims (used in replica exchange)"
        # MD: temperature and rescale velocities
        self._md.set_thermostat(2, 1.0 / (kB * inv_temp), self.md_tau)
        self._md.rescale_velocities(math.sqrt(self.inv_temp / inv_temp))
        # MC: temperature
        self._mc_sigma.set_temperature(1.0 / inv_temp)
        self._mc_gamma.set_temperature(1.0 / inv_temp)
        # store it locally as well
        self.inv_temp = inv_temp

    def get_inv_temp(self):
        return self.inv_temp

    def get_state(self):
        """get_state and set_state work in tandem. They should only respect two
        rules:
            - get_state should return a dict that contains a key called inv_temp
              which is the inverse temperature used by replica-exchange
            - set_state should be able to set the state of this node when given
              the output of a get_state call.
        """
        state = {}
        state['inv_temp'] = self.get_inv_temp()
        state['gamma_mc_stepsize'] = self._nm_gamma.get_sigma()
        state['sigma_mc_stepsize'] = self._nm_sigma.get_sigma()
        return state

    def set_state(self, state):
        """set the state of this node according to the passed dict.
        See get_state.
        """
        self.set_inv_temp(state['inv_temp'])
        self._nm_gamma.set_sigma(state['gamma_mc_stepsize'])
        self._nm_sigma.set_sigma(state['sigma_mc_stepsize'])

if __name__ == '__main__':
    sfo = sfo()
    sfo.init_model('./', 'sequence.dat', 'generated2.pdb',
                   'NOE_HN-full_7A_sparse100.tbl')
    sfo.init_simulation()
    sfo.init_stats()
    for i in xrange(100):
        print("\r%d" % i, end=' ')
        sys.stdout.flush()
        sfo.do_mc(10)
        sfo.do_md(10)
        sfo.write_stats()



## \example container/restrain_in_sphere.py
# This fragment shows how to restrain a set of points stored in a
# SingletonContainer in a sphere of radius 'radius' centered around
# 'center'.

import IMP.example
import sys

IMP.setup_from_argv(sys.argv, "restrain in sphere")

radius = 10
stiffness = 2
center = IMP.algebra.Vector3D(1, 2, 3)
(m, c) = IMP.example.create_model_and_particles()
ub = IMP.core.HarmonicUpperBound(radius, stiffness)

# Restrain based on the distance to a single point (hence a ball)
ss = IMP.core.DistanceToSingletonScore(ub, center)

r = IMP.container.SingletonsRestraint(ss, c)
r.evaluate(False)



#!/usr/bin/env python

"""Parse the manual's XML output to get the contents. Then patch each page
   in the HTML output to add navigation links (next page, previous page,
   edit on GitHub).

   This also applies minor fixes to the HTML to fix places where doxygen
   generates wonky output.
"""

import xml.etree.ElementTree as ET
import glob
import os
import sys
import re


def get_basename_from_xml(f):
    return os.path.basename(f)[:-4]


def get_version():
    version = "develop"
    versionpath = "VERSION"
    if os.path.exists(versionpath):
        with open(versionpath, "r") as fh:
            version = fh.read().strip()
    return version


def get_source_file_name(module, subdir, f, example):
    """Get the relative path to the source for `f` in module `module`."""
    if example:
        return os.path.join('modules', module, 'examples', subdir, f)
    elif f.endswith('.py'):
        return os.path.join('modules', module, 'pyext', 'src', subdir, f)
    else:
        return os.path.join('modules', module, 'include', subdir, f)


def patch_html(content):
    """Fix wonky HTML generated by doxygen"""
    # 1. Cannot use \example in markup since that is a doxygen special
    #    command; but \\example generates '\\example' in the HTML output. So
    #    replace all \\ with \.
    # 2. Have to use %IMP in the text to prevent doxygen auto-linking it
    #    to the IMP namespace docs, but this puts literal %IMP in titles.
    # 3. Trying to show a literal end-comment with `*/` doesn't work.
    for line in content:
        yield line.replace('\\\\', '\\') \
                  .replace('%IMP', 'IMP') \
                  .replace('`` blocks', '<code>*/</code> blocks')


def get_all_listitems(top_element, parent=None):
    """Recursively get all listitem objects in an ElementTree.
       Yield tuples of a listitem object and its immediate parent."""
    for child in top_element:
        if child.tag == 'listitem':
            yield child, parent
            for x in get_all_listitems(child, child):
                yield x
        else:
            for x in get_all_listitems(child, parent):
                yield x


def get_all_links(item):
    """Get all hyperlinks that are children of an ElementTree object."""
    for para in item.findall("para"):
        for ref in para.findall("ref"):
            if ref.attrib['kindref'] == 'compound' \
               and 'external' not in ref.attrib:
                yield ref


def get_page_from_source(source, id_re, id_to_page):
    """Given a source .md or .dox file, return the Page object."""
    with open(source) as fh:
        for line in fh:
            if '\\mainpage' in line:
                return id_to_page['indexpage']
            m = id_re.search(line)
            if m:
                return id_to_page[m.group(1)]
    raise ValueError("Cannot find page id in %s" % source)


class Page(object):
    """Representation of a single page in the documentation"""
    def __init__(self, id):
        self.id = id
        self.next_page = self.prev_page = self.parent_page = None
        self.children = []

    def write_children(self, out):
        """Print out an HTML list of all children of this Page"""
        out.write('<ul>\n')
        for c in self.children:
            out.write('<li><a class="el" href="%s.html">%s</a></li>\n'
                      % (c.out_file_name, c.title))
        out.write('</ul>\n')

    def make_link(self, caption, img, dest):
        """Return an HTML fragment to link to another Page"""
        return '<a href="%s.html" title="%s: %s"><img src="%s.png" ' \
               'alt="%s"/></a>' % (dest.out_file_name, caption, dest.title,
                                   img, img)

    def map_location_to_source(self, top_source_dir):
        """Try to map the XML location to a real file in the source dir"""
        if not hasattr(self, 'location'):
            return
        if 'IMP' in self.location:
            ps = self.location[self.location.find('IMP')+4:].split(os.path.sep)
            example = False
        elif '/examples/' in self.location:
            ps = self.location[
                self.location.find('/examples/')+10:].split(os.path.sep)
            example = True
        else:
            return
        if len(ps) == 1 or ps[0] == 'internal':
            ps.insert(0, 'kernel')
        source = get_source_file_name(ps[0], os.path.sep.join(ps[1:-1]),
                                      ps[-1], example)
        if os.path.exists(os.path.join(top_source_dir, source)):
            self.source_file_name = source

    def github_edit(self):
        """Return an HTML fragment to edit this Page on GitHub"""
        if not hasattr(self, 'source_file_name'):
            return ''
        sp = self.source_file_name.split(os.path.sep)
        if sp[:2] == ['modules', 'pmi']:
            root = 'https://github.com/salilab/pmi/blob/develop/'
            del sp[:2]
        elif sp[:2] == ['modules', 'pmi1']:
            root = 'https://github.com/salilab/pmi/blob/legacy-pmi1/'
            del sp[:2]
        elif sp[:2] == ['modules', 'npctransport']:
            root = 'https://github.com/salilab/npctransport/blob/develop/'
            del sp[:2]
        else:
            root = 'https://github.com/salilab/imp/blob/develop/'
        return('      <li><a href="%s%s"><i class="fab fa-github">'
               '</i> Edit on GitHub</a></li>\n' % (root, os.path.sep.join(sp)))


class Docs(object):
    """Representation of all doxygen documentation"""

    def __init__(self, xml_dir, html_dir, top_source_dir, source_subdirs):
        """Read in all documentation (as a list of Page objects) built from
           sources under top_source_dir and output to XML in xml_dir and
           HTML in html_dir"""
        self.html_dir = html_dir
        self._contents_page = os.path.join(xml_dir, 'indexpage.xml')
        self.pages = self._read_xml_pages(xml_dir)
        self.page_by_id = dict((p.id, p) for p in self.pages)
        self._find_page_source(top_source_dir, source_subdirs)

    def build_hierarchy(self):
        """By inspecting the contents page, determine the child-parent-sibling
           relationships between all pages in the documentation"""
        index_page = self.page_by_id['indexpage']
        # Make sure that all pages not listed in the contents have an 'up'
        # link to the main page
        for p in self.pages:
            if p is not index_page:
                p.parent_page = index_page
        prev_page = index_page
        tree = ET.parse(self._contents_page)
        root = tree.getroot()
        for item, parent in get_all_listitems(root):
            parent_page = index_page
            if parent is not None:
                links = list(get_all_links(parent))
                if len(links) > 0:
                    parent_page = self.page_by_id[links[0].attrib['refid']]
            for link in get_all_links(item):
                this_page = self.page_by_id[link.attrib['refid']]
                if prev_page:
                    this_page.prev_page = prev_page
                    prev_page.next_page = this_page
                this_page.parent_page = parent_page
                parent_page.children.append(this_page)
                prev_page = this_page

    def _read_xml_pages(self, xml_dir):
        """Read all generated doxygen XML files to determine page properties,
           such as title, doxygen ID, and output file name"""
        pages = []
        for f in glob.glob(os.path.join(xml_dir, "*.xml")):
            tree = ET.parse(f)
            root = tree.getroot()
            compounddef = root.find('compounddef')
            if (compounddef is not None
                    and compounddef.attrib['kind'] == 'page'):
                p = Page(compounddef.attrib['id'])
                # Remove .xml suffix
                p.out_file_name = get_basename_from_xml(f)
                # Handle index special case
                if p.out_file_name == 'indexpage':
                    p.out_file_name = 'index'
                p.title = compounddef.find('title').text.replace('%IMP', 'IMP')
                pages.append(p)
            elif (compounddef is not None
                  and compounddef.attrib['kind'] in ('class', 'file')):
                p = Page(compounddef.attrib['id'])
                # Remove .xml suffix
                p.out_file_name = get_basename_from_xml(f)
                p.location = compounddef.find('location').attrib['file']
                if '/examples/' in p.location:
                    p.out_file_name += '-example'
                # Exclude source of .md or .dox files
                if not p.id.endswith('_8md') and not p.id.endswith('_8dox'):
                    pages.append(p)
        return pages

    def _find_page_source(self, top_source_dir, source_subdirs):
        """Determine which source file generated each output
           documentation file"""
        for pattern, id_re in (("*.md", re.compile(r"\{#([^}]+)\}")),
                               ("*.dox", re.compile(r"\\page\s+(\S+)"))):
            for source_subdir in source_subdirs:
                for source in glob.glob(os.path.join(top_source_dir,
                                                     source_subdir, pattern)):
                    # Ignore toplevel README and CONTRIBUTING
                    if source_subdir == '.' \
                       and (source.endswith('README.md')
                            or source.endswith('CONTRIBUTING.md')):
                        continue
                    page = get_page_from_source(source, id_re, self.page_by_id)
                    if source_subdir == '.':
                        page.source_file_name = os.path.basename(source)
                    else:
                        page.source_file_name = os.path.join(
                            source_subdir, os.path.basename(source))

    def get_html_pages(self, page):
        """Return the full path(s) to a page's output HTML file(s), if any"""
        base = page.out_file_name
        pages = self._get_htmls_from_basename(base)
        # Sometimes doxygen makes this substitution, sometimes it doesn't,
        # so try both
        if '__' in base:
            pages.extend(
                self._get_htmls_from_basename(base.replace('__', '_')))
        return pages

    def _get_htmls_from_basename(self, base):
        paths = [os.path.join(self.html_dir, b + '.html')
                 for b in (base, base + '_source')]
        paths = [b for b in paths if os.path.exists(b)]
        if not paths:
            # Try with a namespace prefix
            g = glob.glob(os.path.join(self.html_dir,
                                       "*_2" + base + '.html'))
            if len(g) == 1:
                return g
        return paths

    def add_page_navigation(self, page):
        """Patch the HTML output for a given page to add navigation and other
           useful links, such as GitHub editing"""
        links = []
        if page.prev_page:
            links.append(page.make_link("Previous page", "prev",
                                        page.prev_page))
        if page.parent_page:
            links.append(page.make_link("Parent page", "up",
                                        page.parent_page))
        if page.next_page:
            links.append(page.make_link("Next page", "next",
                                        page.next_page))
        edit_link = page.github_edit()
        doxversion = '<a class="doxversion" ' \
                     + 'href="https://integrativemodeling.org/doc.html">' \
                     + 'version %s</a>' % get_version()
        toplinks = '<div class="doxnavlinks">' + doxversion \
                   + " ".join(links) + '</div>\n'
        botlinks = '<div class="doxnavlinks">' + " ".join(links) + '</div>\n'
        for fname in self.get_html_pages(page):
            with open(fname) as fh:
                content = patch_html(fh.readlines())
            with open(fname, 'w') as out:
                for line in content:
                    if line.startswith('</div><!-- top -->'):
                        out.write(line)
                        out.write(toplinks)
                    elif '<span>Other&#160;Versions</span>' in line:
                        out.write(line)
                        out.write(edit_link)
                    elif line.startswith('<hr class="footer"') and links:
                        out.write('<hr class="footer"/>')
                        out.write(botlinks)
                    elif '[SUBPAGES]' in line:
                        page.write_children(out)
                    else:
                        out.write(line)

    def patch_contents(self):
        """Apply extra patches to index.html"""
        index_html = os.path.join(self.html_dir, 'index.html')
        version = '<b>' + get_version() + '</b>'
        if 'develop' in version:
            version += ' (a nightly build)'
        with open(index_html) as fh:
            content = fh.readlines()
        with open(index_html, 'w') as out:
            for line in content:
                line = line.replace('<div class="title">Contents',
                                    '<div class="title">IMP Manual')
                line = line.replace('IMPVERSION', version)
                out.write(line)


def main():
    docs = Docs(xml_dir='doxygen/manual/xml',
                html_dir='doc/manual',
                top_source_dir=sys.argv[1],
                source_subdirs=sys.argv[2:])
    docs.build_hierarchy()
    for p in docs.pages:
        docs.add_page_navigation(p)
    docs.patch_contents()


if __name__ == '__main__':
    main()



"""@namespace IMP::test
   @brief Methods and classes for testing the IMP kernel and modules.
   @ingroup python
"""

import re
import math
import sys
import os
import tempfile
import random
import IMP
import time
import types
import shutil
import difflib
import pprint
import unittest
from unittest.util import safe_repr
import datetime
import pickle
import contextlib
import subprocess


# Expose some unittest decorators for convenience
expectedFailure = unittest.expectedFailure
skip = unittest.skip
skipIf = unittest.skipIf
skipUnless = unittest.skipUnless


class _TempDir(object):
    def __init__(self, dir=None):
        self.tmpdir = tempfile.mkdtemp(dir=dir)

    def __del__(self):
        shutil.rmtree(self.tmpdir, ignore_errors=True)


@contextlib.contextmanager
def temporary_working_directory():
    """Simple context manager to run in a temporary directory.
       While the context manager is active (within the 'with' block)
       the current working directory is set to a temporary directory.
       When the context manager exists, the working directory is reset
       and the temporary directory deleted."""
    origdir = os.getcwd()
    tmpdir = tempfile.mkdtemp()
    os.chdir(tmpdir)
    yield tmpdir
    os.chdir(origdir)
    shutil.rmtree(tmpdir, ignore_errors=True)


@contextlib.contextmanager
def temporary_directory(dir=None):
    """Simple context manager to make a temporary directory.
       The temporary directory has the same lifetime as the context manager
       (i.e. it is created at the start of the 'with' block, and deleted
       at the end of the block).
       @param dir If given, the temporary directory is made as a subdirectory
                  of that directory, rather than in the default temporary
                  directory location (e.g. /tmp)
       @return the full path to the temporary directory.
    """
    tmpdir = tempfile.mkdtemp(dir=dir)
    yield tmpdir
    shutil.rmtree(tmpdir, ignore_errors=True)


def numerical_derivative(func, val, step):
    """Calculate the derivative of the single-value function `func` at
       point `val`. The derivative is calculated using simple finite
       differences starting with the given `step`; Richardson extrapolation
       is then used to extrapolate the derivative at step=0."""
    maxsteps = 50
    con = 1.4
    safe = 2.0
    err = 1.0e30
    f1 = func(val + step)
    f2 = func(val - step)
    # create first element in triangular matrix d of derivatives
    d = [[(f1 - f2) / (2.0 * step)]]
    retval = None
    for i in range(1, maxsteps):
        d.append([0.] * (i + 1))
        step /= con
        f1 = func(val + step)
        f2 = func(val - step)
        d[i][0] = (f1 - f2) / (2.0 * step)
        fac = con * con
        for j in range(1, i + 1):
            d[i][j] = (d[i][j-1] * fac - d[i-1][j-1]) / (fac - 1.)
            fac *= con * con
            errt = max(abs(d[i][j] - d[i][j-1]),
                       abs(d[i][j] - d[i-1][j-1]))
            if errt <= err:
                err = errt
                retval = d[i][j]
        if abs(d[i][i] - d[i-1][i-1]) >= safe * err:
            break
    if retval is None:
        raise ValueError("Cannot calculate numerical derivative")
    return retval


def xyz_numerical_derivatives(sf, xyz, step):
    """Calculate the x,y and z derivatives of the scoring function `sf`
       on the `xyz` particle. The derivatives are approximated numerically
       using the numerical_derivatives() function."""
    class _XYZDerivativeFunc(object):
        def __init__(self, sf, xyz, basis_vector):
            self._xyz = xyz
            self._sf = sf
            self._basis_vector = basis_vector
            self._starting_coordinates = xyz.get_coordinates()

        def __call__(self, val):
            self._xyz.set_coordinates(self._starting_coordinates +
                                      self._basis_vector * val)
            return self._sf.evaluate(False)

    return tuple([IMP.test.numerical_derivative(
                  _XYZDerivativeFunc(sf, xyz, IMP.algebra.Vector3D(*x)),
                  0, 0.01)
                  for x in ((1, 0, 0), (0, 1, 0), (0, 0, 1))])


class TestCase(unittest.TestCase):
    """Super class for IMP test cases.
       This provides a number of useful IMP-specific methods on top of
       the standard Python `unittest.TestCase` class.
       Test scripts should generally contain a subclass of this class,
       conventionally called `Tests` (this makes it easier to run an
       individual test from the command line) and use IMP::test::main()
       as their main function."""

    # Provide assert(Not)Regex for Python 2 users (assertRegexMatches is
    # deprecated in Python 3)
    if not hasattr(unittest.TestCase, 'assertRegex'):
        assertRegex = unittest.TestCase.assertRegexpMatches
        assertNotRegex = unittest.TestCase.assertNotRegexpMatches

    def __init__(self, *args, **keys):
        unittest.TestCase.__init__(self, *args, **keys)
        self._progname = os.path.abspath(sys.argv[0])

    def setUp(self):
        self.__check_level = IMP.get_check_level()
        # Turn on expensive runtime checks while running the test suite:
        IMP.set_check_level(IMP.USAGE_AND_INTERNAL)
        # python ints are bigger than C++ ones, so we need to make sure it fits
        # otherwise python throws fits
        IMP.random_number_generator.seed(hash(time.time()) % 2**30)

    def tearDown(self):
        # Restore original check level
        IMP.set_check_level(self.__check_level)
        # Clean up any temporary files
        if hasattr(self, '_tmpdir'):
            del self._tmpdir

    def get_input_file_name(self, filename):
        """Get the full name of an input file in the top-level
           test directory."""
        testdir = os.path.dirname(self._progname)
        if self.__module__ != '__main__':
            testdir = os.path.dirname(sys.modules[self.__module__].__file__)
        dirs = testdir.split(os.path.sep)
        for i in range(len(dirs), 0, -1):
            input = os.path.sep.join(dirs[:i] + ['input'])
            if os.path.isdir(input):
                ret = os.path.join(input, filename)
                if not os.path.exists(ret):
                    raise IOError("Test input file "+ret+" does not exist")
                return ret
        raise IOError("No test input directory found")

    def open_input_file(self, filename, mode='rb'):
        """Open and return an input file in the top-level test directory."""
        return open(self.get_input_file_name(filename), mode)

    def get_tmp_file_name(self, filename):
        """Get the full name of an output file in the tmp directory.
           The directory containing this file will be automatically
           cleaned up when the test completes."""
        if not hasattr(self, '_tmpdir'):
            self._tmpdir = _TempDir(os.environ.get('IMP_TMP_DIR'))
        tmpdir = self._tmpdir.tmpdir
        return os.path.join(tmpdir, filename)

    def get_magnitude(self, vector):
        """Get the magnitude of a list of floats"""
        return sum([x*x for x in vector], 0)**.5

    def assertRaisesUsageException(self, c, *args, **keys):
        """Assert that the given callable object raises UsageException.
           This differs from unittest's assertRaises in that the test
           is skipped in fast mode (where usage checks are turned off)."""
        if IMP.get_check_level() >= IMP.USAGE:
            return self.assertRaises(IMP.UsageException, c, *args, **keys)

    def assertRaisesInternalException(self, c, *args, **keys):
        """Assert that the given callable object raises InternalException.
           This differs from unittest's assertRaises in that the test
           is skipped in fast mode (where internal checks are turned off)."""
        if IMP.get_check_level() >= IMP.USAGE_AND_INTERNAL:
            return self.assertRaises(IMP.InternalException, c, *args, **keys)

    def assertNotImplemented(self, c, *args, **keys):
        """Assert that the given callable object is not implemented."""
        return self.assertRaises(IMP.InternalException, c, *args, **keys)

    def assertXYZDerivativesInTolerance(self, sf, xyz, tolerance=0,
                                        percentage=0):
        """Assert that x,y,z analytical derivatives match numerical within
           a tolerance, or a percentage (of the analytical value), whichever
           is larger. `sf` should be a ScoringFunction or Restraint,
           although for backwards compatibility a Model is also accepted."""
        sf.evaluate(True)
        derivs = xyz.get_derivatives()
        num_derivs = xyz_numerical_derivatives(sf, xyz, 0.01)
        pct = percentage / 100.0
        self.assertAlmostEqual(
            self.get_magnitude(derivs-num_derivs), 0,
            delta=tolerance+percentage*self.get_magnitude(num_derivs),
            msg="Don't match "+str(derivs) + str(num_derivs))
        self.assertAlmostEqual(derivs[0], num_derivs[0],
                               delta=max(tolerance, abs(derivs[0]) * pct))
        self.assertAlmostEqual(derivs[1], num_derivs[1],
                               delta=max(tolerance, abs(derivs[1]) * pct))
        self.assertAlmostEqual(derivs[2], num_derivs[2],
                               delta=max(tolerance, abs(derivs[2]) * pct))

    def assertSequenceAlmostEqual(self, first, second, places=None, msg=None,
                                  delta=None):
        """Fail if the difference between any two items in the two sequences
           are exceed the specified number of places or delta. See
           `assertAlmostEqual`.
        """
        if delta is not None and places is not None:
            raise TypeError("specify delta or places not both")

        ftype = type(first)
        ftypename = ftype.__name__
        stype = type(second)
        stypename = stype.__name__
        if ftype != stype:
            raise self.failureException(
                'Sequences are of different types: %s != %s' % (
                    ftypename, stypename))

        try:
            flen = len(first)
        except (NotImplementedError, TypeError):
            raise self.failureException(
                'First %s has no length' % (ftypename))
        try:
            slen = len(second)
        except (NotImplementedError, TypeError):
            raise self.failureException(
                'Second %s has no length' % (stypename))

        if flen != slen:
            raise self.failureException(
                'Sequences have non equal lengths: %d != %d' % (flen, slen))

        differing = None
        for i in range(min(flen, slen)):
            differing = '%ss differ: %s != %s\n' % (
                    ftypename.capitalize(), safe_repr(first),
                    safe_repr(second))

            try:
                f = first[i]
            except (TypeError, IndexError, NotImplementedError):
                differing += ('\nUnable to index element %d of first %s\n' %
                              (i, ftypename))
                break

            try:
                s = second[i]
            except (TypeError, IndexError, NotImplementedError):
                differing += ('\nUnable to index element %d of second %s\n' %
                              (i, stypename))
                break

            try:
                self.assertAlmostEqual(
                    f, s, places=places, msg=msg, delta=delta)
            except (TypeError, ValueError, NotImplementedError,
                    AssertionError):
                differing += (
                    "\nFirst differing element "
                    "%d:\n%s\n%s\n") % (i, safe_repr(f), safe_repr(s))
                break
        else:
            return

        standardMsg = differing
        diffMsg = '\n' + '\n'.join(
            difflib.ndiff(pprint.pformat(first).splitlines(),
                          pprint.pformat(second).splitlines()))
        standardMsg = self._truncateMessage(standardMsg, diffMsg)
        msg = self._formatMessage(msg, standardMsg)
        raise self.failureException(msg)

    def create_point_particle(self, model, x, y, z):
        """Make a particle with optimizable x, y and z attributes, and
           add it to the model."""
        p = IMP.Particle(model)
        p.add_attribute(IMP.FloatKey("x"), x, True)
        p.add_attribute(IMP.FloatKey("y"), y, True)
        p.add_attribute(IMP.FloatKey("z"), z, True)
        return p

    def probabilistic_check(self, testcall, chance_of_failure):
        """Help handle a test which is expected to fail some fraction of
        the time. The test is run multiple times and an exception
        is thrown only if it fails too many times.
        @note Use of this function should be avoided. If there is a corner
              case that results in a test 'occasionally' failing, write a
              new test specifically for that corner case and assert that
              it fails consistently (and remove the corner case from the
              old test).
        """
        prob = chance_of_failure
        tries = 1
        while prob > .001:
            tries += 1
            prob = prob*chance_of_failure
        for i in range(0, tries):
            try:
                eval(testcall)
            except:  # noqa: E722
                pass
            else:
                return
        eval(testcall)
        raise AssertionError("Too many failures")

    def failure_probability(self, testcall):
        """Estimate how likely a given block of code is to raise an
        AssertionError."""
        failures = 0
        tries = 0.0
        while failures < 10 and tries < 1000:
            try:
                eval(testcall)
            except:  # noqa: E722
                failures += 1
            tries = tries+1
        return failures/tries

    def randomize_particles(self, particles, deviation):
        """Randomize the xyz coordinates of a list of particles"""
        # Note: cannot use XYZ here since that pulls in IMP.core
        xkey = IMP.FloatKey("x")
        ykey = IMP.FloatKey("y")
        zkey = IMP.FloatKey("z")
        for p in particles:
            p.set_value(xkey, random.uniform(-deviation, deviation))
            p.set_value(ykey, random.uniform(-deviation, deviation))
            p.set_value(zkey, random.uniform(-deviation, deviation))

    def particle_distance(self, p1, p2):
        """Return distance between two given particles"""
        xkey = IMP.FloatKey("x")
        ykey = IMP.FloatKey("y")
        zkey = IMP.FloatKey("z")
        dx = p1.get_value(xkey) - p2.get_value(xkey)
        dy = p1.get_value(ykey) - p2.get_value(ykey)
        dz = p1.get_value(zkey) - p2.get_value(zkey)
        return math.sqrt(dx*dx + dy*dy + dz*dz)

    def check_unary_function_deriv(self, func, lb, ub, step):
        """Check the unary function func's derivatives against numerical
           approximations between lb and ub"""
        for f in [lb + i * step for i in range(1, int((ub-lb)/step))]:
            (v, d) = func.evaluate_with_derivative(f)
            da = numerical_derivative(func.evaluate, f, step / 10.)
            self.assertAlmostEqual(d, da, delta=max(abs(.1 * d), 0.01))

    def check_unary_function_min(self, func, lb, ub, step, expected_fmin):
        """Make sure that the minimum of the unary function func over the
           range between lb and ub is at expected_fmin"""
        fmin, vmin = lb, func.evaluate(lb)
        for f in [lb + i * step for i in range(1, int((ub-lb)/step))]:
            v = func.evaluate(f)
            if v < vmin:
                fmin, vmin = f, v
        self.assertAlmostEqual(fmin, expected_fmin, delta=step)

    def check_standard_object_methods(self, obj):
        """Check methods that every IMP::Object class should have"""
        obj.set_was_used(True)
        # Test get_from static method
        cls = type(obj)
        self.assertIsNotNone(cls.get_from(obj))
        self.assertRaises(ValueError, cls.get_from, IMP.Model())
        # Test __str__ and __repr__
        self.assertIsInstance(str(obj), str)
        self.assertIsInstance(repr(obj), str)
        # Test get_version_info()
        verinf = obj.get_version_info()
        self.assertIsInstance(verinf, IMP.VersionInfo)
        # Test SWIG thisown flag
        o = obj.thisown
        obj.thisown = o

    def create_particles_in_box(self, model, num=10,
                                lb=[0, 0, 0],
                                ub=[10, 10, 10]):
        """Create a bunch of particles in a box"""
        import IMP.algebra
        lbv = IMP.algebra.Vector3D(lb[0], lb[1], lb[2])
        ubv = IMP.algebra.Vector3D(ub[0], ub[1], ub[2])
        ps = []
        for i in range(0, num):
            v = IMP.algebra.get_random_vector_in(
                IMP.algebra.BoundingBox3D(lbv, ubv))
            p = self.create_point_particle(model, v[0], v[1], v[2])
            ps.append(p)
        return ps

    def _get_type(self, module, name):
        return eval('type('+module+"."+name+')')

    def assertValueObjects(self, module, exceptions_list):
        "Check that all the C++ classes in the module are values or objects."
        all = dir(module)
        ok = set(exceptions_list + module._value_types + module._object_types
                 + module._raii_types + module._plural_types)

        bad = []
        for name in all:
            if self._get_type(module.__name__, name) == type \
                    and not name.startswith("_"):
                if name.find("SwigPyIterator") != -1:
                    continue
                # Exclude Python-only classes
                if not eval('hasattr(%s.%s, "__swig_destroy__")'
                            % (module.__name__, name)):
                    continue
                if name in ok:
                    continue
                bad.append(name)
        self.assertEqual(
            len(bad), 0,
            "All IMP classes should be labeled as values or objects to get "
            "memory management correct in Python. The following are not:\n%s\n"
            "Please add an IMP_SWIG_OBJECT or IMP_SWIG_VALUE call to the "
            "Python wrapper, or if the class has a good reason to be "
            "neither, add the name to the value_object_exceptions list in "
            "the IMPModuleTest call." % str(bad))
        for e in exceptions_list:
            self.assertTrue(
                e not in module._value_types + module._object_types
                + module._raii_types + module._plural_types,
                "Value/Object exception "+e+" is not an exception")

    def _check_spelling(self, word, words):
        """Check that the word is spelled correctly"""
        if "words" not in dir(self):
            with open(IMP.test.get_data_path("linux.words"), "r") as fh:
                wordlist = fh.read().split("\n")
            # why is "all" missing on my mac?
            custom_words = ["info", "prechange", "int", "ints", "optimizeds",
                            "graphviz", "voxel", "voxels", "endian", 'rna',
                            'dna', "xyzr", "pdbs", "fft", "ccc", "gaussian"]
            # Exclude some common alternative spellings - we want to
            # be consistent
            exclude_words = set(["adapter", "grey"])
            self.words = set(wordlist+custom_words) - exclude_words
        if self.words:
            for i in "0123456789":
                if i in word:
                    return True
            if word in words:
                return True
            if word in self.words:
                return True
            else:
                return False
        else:
            return True

    def assertClassNames(self, module, exceptions, words):
        """Check that all the classes in the module follow the IMP
           naming conventions."""
        all = dir(module)
        misspelled = []
        bad = []
        cc = re.compile("([A-Z][a-z]*)")
        for name in all:
            if self._get_type(module.__name__, name) == type \
                    and not name.startswith("_"):
                if name.find("SwigPyIterator") != -1:
                    continue
                for t in re.findall(cc, name):
                    if not self._check_spelling(t.lower(), words):
                        misspelled.append(t.lower())
                        bad.append(name)

        self.assertEqual(
            len(bad), 0,
            "All IMP classes should be properly spelled. The following "
            "are not: %s.\nMisspelled words: %s. Add words to the "
            "spelling_exceptions variable of the IMPModuleTest if needed."
            % (str(bad), ", ".join(set(misspelled))))

        for name in all:
            if self._get_type(module.__name__, name) == type \
                    and not name.startswith("_"):
                if name.find("SwigPyIterator") != -1:
                    continue
                if name.find('_') != -1:
                    bad.append(name)
                if name.lower == name:
                    bad.append(name)
                for t in re.findall(cc, name):
                    if not self._check_spelling(t.lower(), words):
                        print("misspelled %s in %s" % (t, name))
                        bad.append(name)
        self.assertEqual(
            len(bad), 0,
            "All IMP classes should have CamelCase names. The following "
            "do not: %s." % "\n".join(bad))

    def _check_function_name(self, prefix, name, verbs, all, exceptions, words,
                             misspelled):
        if prefix:
            fullname = prefix+"."+name
        else:
            fullname = name
        old_exceptions = [
            'unprotected_evaluate', "unprotected_evaluate_if_good",
            "unprotected_evaluate_if_below",
            "after_evaluate", "before_evaluate", "has_attribute",
            "decorate_particle", "particle_is_instance"]
        if name in old_exceptions:
            return []
        if fullname in exceptions:
            return []
        if name.endswith("swigregister"):
            return []
        if name.lower() != name:
            if name[0].lower() != name[0] and name.split('_')[0] in all:
                # static methods
                return []
            else:
                return [fullname]
        tokens = name.split("_")
        if tokens[0] not in verbs:
            return [fullname]
        for t in tokens:
            if not self._check_spelling(t, words):
                misspelled.append(t)
                print("misspelled %s in %s" % (t, name))
                return [fullname]
        return []

    def _static_method(self, module, prefix, name):
        """For static methods of the form Foo.bar SWIG creates free functions
           named Foo_bar. Exclude these from spelling checks since the method
           Foo.bar has already been checked."""
        if prefix is None and '_' in name:
            modobj = eval(module)
            cls, meth = name.split('_', 1)
            if hasattr(modobj, cls):
                clsobj = eval(module + '.' + cls)
                if hasattr(clsobj, meth):
                    return True

    def _check_function_names(self, module, prefix, names, verbs, all,
                              exceptions, words, misspelled):
        bad = []
        for name in names:
            typ = self._get_type(module, name)
            if name.startswith("_") or name == "weakref_proxy":
                continue
            if typ in (types.BuiltinMethodType, types.MethodType) \
                    or (typ == types.FunctionType and  # noqa: E721
                        not self._static_method(module, prefix, name)):
                bad.extend(self._check_function_name(prefix, name, verbs, all,
                                                     exceptions, words,
                                                     misspelled))
            if typ == type and "SwigPyIterator" not in name:
                members = eval("dir("+module+"."+name+")")
                bad.extend(self._check_function_names(module+"."+name,
                                                      name, members, verbs, [],
                                                      exceptions, words,
                                                      misspelled))
        return bad

    def assertFunctionNames(self, module, exceptions, words):
        """Check that all the functions in the module follow the IMP
           naming conventions."""
        all = dir(module)
        verbs = set(["add", "remove", "get", "set", "evaluate", "compute",
                     "show", "create", "destroy", "push", "pop", "write",
                     "read", "do", "show", "load", "save", "reset", "accept",
                     "reject", "clear", "handle", "update", "apply",
                     "optimize", "reserve", "dump", "propose", "setup",
                     "teardown", "visit", "find", "run", "swap", "link",
                     "validate"])
        misspelled = []
        bad = self._check_function_names(module.__name__, None, all, verbs,
                                         all, exceptions, words, misspelled)
        message = ("All IMP methods and functions should have lower case "
                   "names separated by underscores and beginning with a "
                   "verb, preferably one of ['add', 'remove', 'get', 'set', "
                   "'create', 'destroy']. Each of the words should be a "
                   "properly spelled English word. The following do not "
                   "(given our limited list of verbs that we check for):\n"
                   "%(bad)s\nIf there is a good reason for them not to "
                   "(eg it does start with a verb, just one with a meaning "
                   "that is not covered by the normal list), add them to the "
                   "function_name_exceptions variable in the "
                   "standards_exceptions file. Otherwise, please fix. "
                   "The current verb list is %(verbs)s"
                   % {"bad": "\n".join(bad), "verbs": verbs})
        if len(misspelled) > 0:
            message += "\nMisspelled words: " + ", ".join(set(misspelled)) \
                       + ". Add words to the spelling_exceptions variable " \
                       + "of the standards_exceptions file if needed."
        self.assertEqual(len(bad), 0, message)

    def assertShow(self, modulename, exceptions):
        """Check that all the classes in modulename have a show method"""
        all = dir(modulename)
        not_found = []
        for f in all:
            # Exclude SWIG C global variables object
            if f == 'cvar':
                continue
            # Exclude Python-only classes; they are all showable
            if not eval('hasattr(%s.%s, "__swig_destroy__")'
                        % (modulename.__name__, f)):
                continue
            if self._get_type(modulename.__name__, f) == type \
                    and not f.startswith("_") \
                    and not f.endswith("_swigregister")\
                    and f not in exceptions\
                    and not f.endswith("Temp") and not f.endswith("Iterator")\
                    and not f.endswith("Exception") and\
                    f not in modulename._raii_types and \
                    f not in modulename._plural_types:
                if not hasattr(getattr(modulename, f), 'show'):
                    not_found.append(f)
        self.assertEqual(
            len(not_found), 0,
            "All IMP classes should support show and __str__. The following "
            "do not:\n%s\n If there is a good reason for them not to, add "
            "them to the show_exceptions variable in the IMPModuleTest "
            "call. Otherwise, please fix." % "\n".join(not_found))
        for e in exceptions:
            self.assertIn(e, all,
                          "Show exception "+e+" is not a class in module")
            self.assertTrue(not hasattr(getattr(modulename, e), 'show'),
                            "Exception "+e+" is not really a show exception")

    def run_example(self, filename):
        """Run the named example script.
           @return a dictionary of all the script's global variables.
                   This can be queried in a test case to make sure
                   the example performed correctly."""
        class _FatalError(Exception):
            pass

        # Add directory containing the example to sys.path, so it can import
        # other Python modules in the same directory
        path, name = os.path.split(filename)
        oldsyspath = sys.path[:]
        olssysargv = sys.argv[:]
        sys.path.insert(0, path)
        sys.argv = [filename]
        vars = {}
        try:
            try:
                exec(open(filename).read(), vars)
            # Catch sys.exit() called from within the example; a non-zero exit
            # value should cause the test case to fail
            except SystemExit as e:
                if e.code != 0 and e.code is not None:
                    raise _FatalError(
                        "Example exit with code %s" % str(e.code))
        finally:
            # Restore sys.path (note that Python 2.3 does not allow
            # try/except/finally, so we need to use nested trys)
            sys.path = oldsyspath
            sys.argv = olssysargv

        return _ExecDictProxy(vars)

    def run_python_module(self, module, args):
        """Run a Python module as if with "python -m <modname>",
           with the given list of arguments as sys.argv.

           If module is an already-imported Python module, run its 'main'
           function and return the result.

           If module is a string, run the module in a subprocess and return
           a subprocess.Popen-like object containing the child stdin,
           stdout and stderr.
        """
        def mock_setup_from_argv(*args, **kwargs):
            # do-nothing replacement for boost command line parser
            pass
        if type(module) == type(os):
            mod = module
        else:
            mod = __import__(module, {}, {}, [''])
        modpath = mod.__file__
        if modpath.endswith('.pyc'):
            modpath = modpath[:-1]
        if type(module) == type(os):
            old_sys_argv = sys.argv
            # boost parser doesn't like being called multiple times per process
            old_setup = IMP.setup_from_argv
            IMP.setup_from_argv = mock_setup_from_argv
            try:
                sys.argv = [modpath] + args
                return module.main()
            finally:
                IMP.setup_from_argv = old_setup
                sys.argv = old_sys_argv
        else:
            return _SubprocessWrapper(sys.executable, [modpath] + args)

    def check_runnable_python_module(self, module):
        """Check a Python module designed to be runnable with 'python -m'
           to make sure it supports standard command line options."""
        # --help should return with exit 0, no errors
        r = self.run_python_module(module, ['--help'])
        out, err = r.communicate()
        self.assertEqual(r.returncode, 0)
        self.assertNotEqual(err, "")
        self.assertEqual(out, "")


class _ExecDictProxy(object):
    """exec returns a Python dictionary, which contains IMP objects, other
       Python objects, as well as base Python modules (such as sys and
       __builtins__). If we just delete this dictionary, it is entirely
       possible that base Python modules are removed from the dictionary
       *before* some IMP objects. This will prevent the IMP objects' Python
       destructors from running properly, so C++ objects will not be
       cleaned up. This class proxies the base dict class, and on deletion
       attempts to remove keys from the dictionary in an order that allows
       IMP destructors to fire."""
    def __init__(self, d):
        self._d = d

    def __del__(self):
        # Try to release example objects in a sensible order
        module_type = type(IMP)
        d = self._d
        for k in d.keys():
            if type(d[k]) != module_type:
                del d[k]

    for meth in ['__contains__', '__getitem__', '__iter__', '__len__',
                 'get', 'has_key', 'items', 'keys', 'values']:
        exec("def %s(self, *args, **keys): "
             "return self._d.%s(*args, **keys)" % (meth, meth))


class _TestResult(unittest.TextTestResult):

    def __init__(self, stream=None, descriptions=None, verbosity=None):
        super(_TestResult, self).__init__(stream, descriptions, verbosity)
        self.all_tests = []

    def stopTestRun(self):
        if 'IMP_TEST_DETAIL_DIR' in os.environ:
            fname = os.path.join(os.environ['IMP_TEST_DETAIL_DIR'],
                                 os.path.basename(sys.argv[0]))
            with open(fname, 'wb') as fh:
                pickle.dump(self.all_tests, fh, -1)
        super(_TestResult, self).stopTestRun()

    def startTest(self, test):
        super(_TestResult, self).startTest(test)
        test.start_time = datetime.datetime.now()

    def _test_finished(self, test, state, detail=None):
        delta = datetime.datetime.now() - test.start_time
        try:
            pv = delta.total_seconds()
        except AttributeError:
            pv = (float(delta.microseconds)
                  + (delta.seconds + delta.days * 24 * 3600) * 10**6) / 10**6
        if pv > 1:
            self.stream.write("in %.3fs ... " % pv)
        if detail is not None and not isinstance(detail, str):
            detail = self._exc_info_to_string(detail, test)
        test_doc = self.getDescription(test)
        test_name = test.id()
        if test_name.startswith('__main__.'):
            test_name = test_name[9:]
        self.all_tests.append({'name': test_name,
                               'docstring': test_doc,
                               'time': pv, 'state': state, 'detail': detail})

    def addSuccess(self, test):
        self._test_finished(test, 'OK')
        super(_TestResult, self).addSuccess(test)

    def addError(self, test, err):
        self._test_finished(test, 'ERROR', err)
        super(_TestResult, self).addError(test, err)

    def addFailure(self, test, err):
        self._test_finished(test, 'FAIL', err)
        super(_TestResult, self).addFailure(test, err)

    def addSkip(self, test, reason):
        self._test_finished(test, 'SKIP', reason)
        super(_TestResult, self).addSkip(test, reason)

    def addExpectedFailure(self, test, err):
        self._test_finished(test, 'EXPFAIL', err)
        super(_TestResult, self).addExpectedFailure(test, err)

    def addUnexpectedSuccess(self, test):
        self._test_finished(test, 'UNEXPSUC')
        super(_TestResult, self).addUnexpectedSuccess(test)

    def getDescription(self, test):
        doc_first_line = test.shortDescription()
        if self.descriptions and doc_first_line:
            return doc_first_line
        else:
            return str(test)


class _TestRunner(unittest.TextTestRunner):
    def _makeResult(self):
        return _TestResult(self.stream, self.descriptions, self.verbosity)


def main(*args, **keys):
    """Run a set of tests; similar to unittest.main().
       Obviates the need to separately import the 'unittest' module, and
       ensures that main() is from the same unittest module that the
       IMP.test testcases are. In addition, turns on some extra checks
       (e.g. trying to use deprecated code will cause an exception
       to be thrown)."""
    import IMP
    IMP.set_deprecation_exceptions(True)
    return unittest.main(testRunner=_TestRunner, *args, **keys)


class _SubprocessWrapper(subprocess.Popen):
    def __init__(self, app, args, cwd=None):
        # For (non-Python) applications to work on Windows, the
        # PATH must include the directory containing built DLLs
        if sys.platform == 'win32' and app != sys.executable:
            # Hack to find the location of build/lib/
            libdir = os.environ['PYTHONPATH'].split(';')[0]
            env = os.environ.copy()
            env['PATH'] += ';' + libdir
        else:
            env = None
        subprocess.Popen.__init__(self, [app]+list(args),
                                  stdin=subprocess.PIPE,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE, env=env, cwd=cwd,
                                  universal_newlines=True)


class ApplicationTestCase(TestCase):
    """Super class for simple IMP application test cases"""
    def _get_application_file_name(self, filename):
        # If we ran from run-all-tests.py, it set an env variable for us with
        # the top-level test directory
        if sys.platform == 'win32':
            filename += '.exe'
        return filename

    def run_application(self, app, args, cwd=None):
        """Run an application with the given list of arguments.
           @return a subprocess.Popen-like object containing the child stdin,
                   stdout and stderr.
        """
        filename = self._get_application_file_name(app)
        if sys.platform == 'win32':
            # Cannot rely on PATH on wine builds, so use full pathname
            return _SubprocessWrapper(os.path.join(os.environ['IMP_BIN_DIR'],
                                                   filename), args, cwd=cwd)
        else:
            return _SubprocessWrapper(filename, args, cwd=cwd)

    def run_python_application(self, app, args):
        """Run a Python application with the given list of arguments.
           The Python application should be self-runnable (i.e. it should
           be executable and with a #! on the first line).
           @return a subprocess.Popen-like object containing the child stdin,
                   stdout and stderr.
        """
        # Handle platforms where /usr/bin/python doesn't work
        if sys.executable != '/usr/bin/python' and 'IMP_BIN_DIR' in os.environ:
            return _SubprocessWrapper(
                sys.executable,
                [os.path.join(os.environ['IMP_BIN_DIR'], app)] + args)
        else:
            return _SubprocessWrapper(app, args)

    def import_python_application(self, app):
        """Import an installed Python application, rather than running it.
           This is useful to directly test components of the application.
           @return the Python module object."""
        try:
            import importlib.machinery
            imp = None
        except ImportError:
            import imp
        name = os.path.splitext(app)[0]
        pathname = os.path.join(os.environ['IMP_BIN_DIR'], app)
        if imp is None:
            return importlib.machinery.SourceFileLoader(name,
                                                        pathname).load_module()
        else:
            return imp.load_source(name, pathname)

    def run_script(self, app, args):
        """Run an application with the given list of arguments.
           @return a subprocess.Popen-like object containing the child stdin,
                   stdout and stderr.
        """
        return _SubprocessWrapper(sys.executable, [app]+args)

    def assertApplicationExitedCleanly(self, ret, error):
        """Assert that the application exited cleanly (return value = 0)."""
        if ret < 0:
            raise OSError("Application exited with signal %d\n" % -ret
                          + error)
        else:
            self.assertEqual(
                ret, 0,
                "Application exited uncleanly, with exit code %d\n" % ret
                + error)

    def read_shell_commands(self, doxfile):
        """Read and return a set of shell commands from a doxygen file.
           Each command is assumed to be in a \\code{.sh}...\\endcode block.
           The doxygen file is specified relative to the test file itself.
           This is used to make sure the commands shown in an application
           example actually work (the testcase can also check the resulting
           files for correctness)."""
        def win32_normpath(p):
            # Sometimes Windows can read Unix-style paths, but sometimes it
            # gets confused... so normalize all paths to be sure
            return " ".join([os.path.normpath(x) for x in p.split()])

        def fix_win32_command(cmd):
            # Make substitutions so a Unix shell command works on Windows
            if cmd.startswith('cp -r '):
                return 'xcopy /E ' + win32_normpath(cmd[6:])
            elif cmd.startswith('cp '):
                return 'copy ' + win32_normpath(cmd[3:])
            else:
                return cmd
        d = os.path.dirname(sys.argv[0])
        doc = os.path.join(d, doxfile)
        inline = False
        cmds = []
        example_path = os.path.abspath(IMP.get_example_path('..'))
        with open(doc) as fh:
            for line in fh.readlines():
                if '\\code{.sh}' in line:
                    inline = True
                elif '\\endcode' in line:
                    inline = False
                elif inline:
                    cmds.append(line.rstrip('\r\n').replace(
                                  '<imp_example_path>', example_path))
        if sys.platform == 'win32':
            cmds = [fix_win32_command(x) for x in cmds]
        return cmds

    def run_shell_command(self, cmd):
        "Print and run a shell command, as returned by read_shell_commands()"
        print(cmd)
        p = subprocess.call(cmd, shell=True)
        if p != 0:
            raise OSError("%s failed with exit value %d" % (cmd, p))


class RefCountChecker(object):
    """Check to make sure the number of C++ object references is as expected"""

    def __init__(self, testcase):
        # Make sure no director objects are hanging around; otherwise these
        # may be unexpectedly garbage collected later, decreasing the
        # live object count
        IMP._director_objects.cleanup()
        self.__testcase = testcase
        if IMP.get_check_level() >= IMP.USAGE_AND_INTERNAL:
            self.__basenum = IMP.Object.get_number_of_live_objects()
            self.__names = IMP.get_live_object_names()

    def assert_number(self, expected):
        "Make sure that the number of references matches the expected value."
        t = self.__testcase
        IMP._director_objects.cleanup()
        if IMP.get_check_level() >= IMP.USAGE_AND_INTERNAL:
            newnames = [x for x in IMP.get_live_object_names()
                        if x not in self.__names]
            newnum = IMP.Object.get_number_of_live_objects()-self.__basenum
            t.assertEqual(newnum, expected,
                          "Number of objects don't match: "
                          + str(newnum) + " != " + str(expected) + " found "
                          + str(newnames))


class DirectorObjectChecker(object):
    """Check to make sure the number of director references is as expected"""

    def __init__(self, testcase):
        IMP._director_objects.cleanup()
        self.__testcase = testcase
        self.__basenum = IMP._director_objects.get_object_count()

    def assert_number(self, expected, force_cleanup=True):
        """Make sure that the number of references matches the expected value.
           If force_cleanup is set, clean up any unused references first before
           doing the assertion.
        """
        t = self.__testcase
        if force_cleanup:
            IMP._director_objects.cleanup()
        t.assertEqual(IMP._director_objects.get_object_count()
                      - self.__basenum, expected)


# Make sure that the IMP binary directory (build/bin) is in the PATH, if
# we're running under wine (the imppy.sh script normally ensures this, but
# wine overrides the PATH). This is needed so that tests of imported Python
# applications can successfully spawn C++ applications (e.g. idock.py tries
# to run recompute_zscore.exe). build/lib also needs to be in the PATH, since
# that's how Windows locates dependent DLLs such as libimp.dll.
if sys.platform == 'win32' and 'PYTHONPATH' in os.environ \
   and 'IMP_BIN_DIR' in os.environ:
    libdir = os.environ['PYTHONPATH'].split(';')[0]
    bindir = os.environ['IMP_BIN_DIR']
    path = os.environ['PATH']
    if libdir not in path or bindir not in path:
        os.environ['PATH'] = bindir + ';' + libdir + ';' + path


__version__ = "2.15.0"

def get_module_version():
    '''Return the version of this module, as a string'''
    return "2.15.0"

def get_module_name():
    '''Return the fully-qualified name of this module'''
    return "IMP::test"

def get_data_path(fname):
    '''Return the full path to one of this module's data files'''
    import IMP
    return IMP._get_module_data_path("test", fname)

def get_example_path(fname):
    '''Return the full path to one of this module's example files'''
    import IMP
    return IMP._get_module_example_path("test", fname)
